import pytest
from insights.parsers import getsebool, SkipException
from insights.tests import context_wrap


GETSEBOOL = """
varnishd_connect_any --> off
virt_read_qemu_ga_data --> off
virt_rw_qemu_ga_data --> off
virt_sandbox_use_all_caps --> on
virt_sandbox_use_audit --> on
virt_sandbox_use_fusefs --> off
virt_sandbox_use_mknod --> off
virt_sandbox_use_netlink --> off
virt_sandbox_use_sys_admin --> off
virt_transition_userdomain --> off
virt_use_comm --> off
virt_use_execmem --> off
virt_use_fusefs --> off
virt_use_glusterd --> off
virt_use_nfs --> on
virt_use_rawip --> off
virt_use_samba --> off
virt_use_sanlock --> off
virt_use_usb --> on
virt_use_xserver --> off
""".strip()

SELINUX_DISABLED = '/usr/sbin/getsebool:  SELinux is disabled'


def test_getsebool():
    result = getsebool.Getsebool(context_wrap(GETSEBOOL))
    assert result["varnishd_connect_any"] == 'off'
    assert result["virt_use_nfs"] == 'on'
    assert "virt_use_xserver" in result
    assert "not_exist_key" not in result


def test_getsebool_disabled():
    with pytest.raises(SkipException) as excinfo:
        getsebool.Getsebool(context_wrap(SELINUX_DISABLED))
    assert 'SELinux is disabled' in str(excinfo.value)



import doctest
import pytest

from insights.combiners import smt
from insights.combiners.smt import CpuTopology
from insights.parsers.smt import CpuCoreOnline, CpuSiblings
from insights.tests import context_wrap


# Path for core online files
ONLINE_PATH = "/sys/devices/system/cpu/cpu{0}/online"
# Path for core siblings files
SIBLINGS_PATH = "/sys/devices/system/cpu/cpu{0}/topology/thread_siblings_list"


@pytest.fixture
def cpu_all_online():
    return [
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(0))),
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(1))),
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(2))),
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(3)))
    ]


def test_hyperthreading_all_online(cpu_all_online):
    siblings = [
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(0))),
        CpuSiblings(context_wrap("1,3", path=SIBLINGS_PATH.format(1))),
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(2))),
        CpuSiblings(context_wrap("1,3", path=SIBLINGS_PATH.format(3)))
    ]

    cpu_topology = CpuTopology(cpu_all_online, siblings)
    assert cpu_topology.online(0)
    assert cpu_topology.siblings(0) == [0, 2]
    assert cpu_topology.online(1)
    assert cpu_topology.siblings(1) == [1, 3]
    assert cpu_topology.online(2)
    assert cpu_topology.siblings(2) == [0, 2]
    assert cpu_topology.online(3)
    assert cpu_topology.siblings(3) == [1, 3]
    assert not cpu_topology.all_solitary


def test_hyperthreading_some_online():
    online = [
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(0))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(1))),
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(2))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(3)))
    ]
    siblings = [
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(0))),
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(2)))
    ]

    cpu_topology = CpuTopology(online, siblings)
    assert cpu_topology.online(0)
    assert cpu_topology.siblings(0) == [0, 2]
    assert not cpu_topology.online(1)
    assert cpu_topology.siblings(1) == []
    assert cpu_topology.online(2)
    assert cpu_topology.siblings(2) == [0, 2]
    assert not cpu_topology.online(3)
    assert cpu_topology.siblings(3) == []
    assert not cpu_topology.all_solitary


def test_without_hyperthreading_all_online(cpu_all_online):
    siblings = [
        CpuSiblings(context_wrap("0", path=SIBLINGS_PATH.format(0))),
        CpuSiblings(context_wrap("1", path=SIBLINGS_PATH.format(1))),
        CpuSiblings(context_wrap("2", path=SIBLINGS_PATH.format(2))),
        CpuSiblings(context_wrap("3", path=SIBLINGS_PATH.format(3)))
    ]

    cpu_topology = CpuTopology(cpu_all_online, siblings)
    assert cpu_topology.online(0)
    assert cpu_topology.siblings(0) == [0]
    assert cpu_topology.online(1)
    assert cpu_topology.siblings(1) == [1]
    assert cpu_topology.online(2)
    assert cpu_topology.siblings(2) == [2]
    assert cpu_topology.online(3)
    assert cpu_topology.siblings(3) == [3]
    assert cpu_topology.all_solitary


def test_without_hyperthreading_some_online():
    online = [
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(0))),
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(1))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(2))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(3)))
    ]
    siblings = [
        CpuSiblings(context_wrap("0", path=SIBLINGS_PATH.format(0))),
        CpuSiblings(context_wrap("1", path=SIBLINGS_PATH.format(1)))
    ]

    cpu_topology = CpuTopology(online, siblings)
    assert cpu_topology.online(0)
    assert cpu_topology.siblings(0) == [0]
    assert cpu_topology.online(1)
    assert cpu_topology.siblings(1) == [1]
    assert not cpu_topology.online(2)
    assert cpu_topology.siblings(2) == []
    assert not cpu_topology.online(3)
    assert cpu_topology.siblings(3) == []
    assert cpu_topology.all_solitary


def test_wrong_index():
    online = [
        CpuCoreOnline(context_wrap("1", path=ONLINE_PATH.format(0))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(1))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(2))),
        CpuCoreOnline(context_wrap("0", path=ONLINE_PATH.format(3)))
    ]
    siblings = [
        CpuSiblings(context_wrap("0", path=SIBLINGS_PATH.format(0))),
    ]

    c = CpuTopology(online, siblings)
    assert c.online(-1) is None
    assert c.siblings(-1) is None
    assert c.online(4) is None
    assert c.siblings(4) is None


def test_doc_examples(cpu_all_online):
    siblings = [
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(0))),
        CpuSiblings(context_wrap("1,3", path=SIBLINGS_PATH.format(1))),
        CpuSiblings(context_wrap("0,2", path=SIBLINGS_PATH.format(2))),
        CpuSiblings(context_wrap("1,3", path=SIBLINGS_PATH.format(3)))
    ]

    env = {
        "cpu_topology": CpuTopology(cpu_all_online, siblings),
    }
    failed, total = doctest.testmod(smt, globs=env)
    assert failed == 0



from insights import dr, rule, make_fail, make_pass, make_fingerprint
from insights.core.plugins import component, Response
from insights.core.evaluators import InsightsEvaluator, SingleEvaluator
from insights.combiners.hostname import hostname
from insights.specs import Specs
from insights.tests import context_wrap


class make_unsure(Response):
    """
    Show that anybody can make a Response subclass for rules to return
    and have it included in the evaluator results.
    """
    response_type = "unsure"
    key_name = "unsure_key"


@component()
def one():
    return 1


@component()
def two():
    return 2


@component()
def boom():
    raise Exception()


@rule()
def report_pass():
    return make_pass("PASS")


@rule()
def report_fail():
    return make_fail("FAIL")


@rule()
def report_fingerprint():
    return make_fingerprint("FINGERPRINT")


@rule()
def report_unsure():
    return make_unsure("unsure")


@rule(boom)
def report_boom():
    pass


@rule(one, two)
def report(a, b):
    return make_fail("FAIL2", a=a, b=b, c=a + b)


@rule(links={"kcs": ["https://www.example.com"]})
def show_links():
    return make_fail("LINKS")


components = [
    hostname,
    Specs.redhat_release,
    Specs.machine_id,
    one,
    two,
    boom,
    report_pass,
    report_fail,
    report_fingerprint,
    report_unsure,
    report_boom,
    report
]


def test_single_evaluator():
    broker = dr.Broker()
    e = SingleEvaluator(broker)
    graph = dr.get_dependency_graph(report)
    result1 = e.process(graph)

    broker = dr.Broker()
    with SingleEvaluator(broker) as e:
        dr.run(report, broker=broker)
        result2 = e.get_response()
        assert result1["reports"] == result2["reports"]
        for k in ["start", "finish", "execution_context", "plugin_sets"]:
            assert k in result1["analysis_metadata"]
            assert k in result2["analysis_metadata"]


def test_insights_evaluator():
    broker = dr.Broker()
    e = InsightsEvaluator(broker)
    graph = dr.get_dependency_graph(report)
    result1 = e.process(graph)

    broker = dr.Broker()
    with InsightsEvaluator(broker) as e:
        dr.run(report, broker=broker)
        result2 = e.get_response()
        assert result1["reports"] == result2["reports"]
        for k in ["start", "finish", "execution_context", "plugin_sets"]:
            assert k in result1["analysis_metadata"]
            assert k in result2["analysis_metadata"]


def test_insights_evaluator_attrs_serial():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    with InsightsEvaluator(broker) as e:
        dr.run(components, broker=broker)
        result = e.get_response()
        assert result["system"]["hostname"] == "www.example.com"
        assert result["system"]["system_id"] == "12345"
        assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"


def test_insights_evaluator_attrs_serial_process():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"


def test_insights_evaluator_attrs_incremental():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    with InsightsEvaluator(broker) as e:
        list(dr.run_incremental(components, broker=broker))
        result = e.get_response()
        assert result["system"]["hostname"] == "www.example.com"
        assert result["system"]["system_id"] == "12345"
        assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"


def test_insights_evaluator_attrs_incremental_process():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"


def test_insights_evaluator_make_fail():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"
    assert len(result["reports"]) == 2
    assert len([r["component"] for r in result["reports"]]) == 2
    assert len([r["type"] for r in result["reports"]]) == 2
    assert len([r["key"] for r in result["reports"]]) == 2


def test_insights_evaluator_make_pass():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"
    assert len(result["pass"]) == 1
    assert len([r["component"] for r in result["pass"]]) == 1
    assert len([r["type"] for r in result["pass"]]) == 1
    assert len([r["key"] for r in result["pass"]]) == 1


def test_insights_evaluator_make_fingerprint():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"
    assert len(result["fingerprints"]) == 1
    assert len([r["component"] for r in result["fingerprints"]]) == 1
    assert len([r["type"] for r in result["fingerprints"]]) == 1
    assert len([r["key"] for r in result["fingerprints"]]) == 1


def test_insights_evaluator_make_unsure():
    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")
    e = InsightsEvaluator(broker)
    e.process(components)
    result = e.get_response()
    assert result["system"]["hostname"] == "www.example.com"
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"
    assert len(result["unsure"]) == 1
    assert len([r["component"] for r in result["unsure"]]) == 1
    assert len([r["type"] for r in result["unsure"]]) == 1
    assert len([r["key"] for r in result["unsure"]]) == 1


def test_insights_evaluator_show_links():
    components = [
        hostname,
        Specs.redhat_release,
        Specs.machine_id,
        show_links,
    ]

    broker = dr.Broker()
    broker[Specs.hostname] = context_wrap("www.example.com")
    broker[Specs.machine_id] = context_wrap("12345")
    broker[Specs.redhat_release] = context_wrap("Red Hat Enterprise Linux Server release 7.4 (Maipo)")

    e = InsightsEvaluator(broker)
    e.process(components)

    result = e.get_response()

    assert result["system"]["hostname"] == "www.example.com", result["system"]["hostname"]
    assert result["system"]["system_id"] == "12345"
    assert result["system"]["metadata"]["release"] == "Red Hat Enterprise Linux Server release 7.4 (Maipo)"

    rule_response = result["reports"][0]
    assert "kcs" in rule_response["links"]



"""
UpstartInitctlList - Command ``initctl --system list``
======================================================

Parser to parse the output of ``initctl --system list`` command.

"""

from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.initctl_lst)
class UpstartInitctlList(CommandParser):
    """
    Class to parse the output of initctl command. It allows a
    system administrator to communicate and interact with the
    Upstart init(8) daemon and list the services managed by
    Upstart init.

    Sample output::

        rc stop/waiting
        vmware-tools start/running
        tty (/dev/tty3) start/running, process 9499
        tty (/dev/tty2) start/running, process 9495
        tty (/dev/tty1) start/running, process 9493
        tty (/dev/tty6) start/running, process 9507
        tty (/dev/tty5) start/running, process 9505
        tty (/dev/tty4) start/running, process 9502
        tty (/dev/ttyS0) start/running, process 9509
        plymouth-shutdown stop/waiting
        control-alt-delete stop/waiting
        rcS-emergency stop/waiting
        readahead-collector stop/waiting
        kexec-disable stop/waiting
        quit-plymouth stop/waiting
        rcS stop/waiting
        prefdm stop/waiting
        init-system-dbus stop/waiting
        ck-log-system-restart stop/waiting
        readahead stop/waiting
        ck-log-system-start stop/waiting
        splash-manager stop/waiting
        start-ttys stop/waiting
        readahead-disable-services stop/waiting
        ck-log-system-stop stop/waiting
        rcS-sulogin stop/waiting
        serial stop/waiting

    Raises:
        SkipException: When nothing need to parse.

    Attributes:
        data(list): Daemon details are stored as list of str.
        tty(dict): Device details are stored with status and process id.
        daemon_proc(dict): Daemon status are stored with its `dameon` name and `status`.

    Examples:
        >>> type(upstart_obj)
        <class 'insights.parsers.upstart.UpstartInitctlList'>
        >>> upstart_obj.upstart_managed('vmware-tools')
        'vmware-tools start/running'
        >>> upstart_obj.daemon_status('vmware-tools')
        'start/running'
        >>> upstart_obj.daemon_status('start-ttys')
        'stop/waiting'
        >>> upstart_obj.dev_status('/dev/tty4')
        'stop/waiting'
        >>> upstart_obj.upstart_managed('/dev/tty3')
        'tty (/dev/tty3) start/running, process 9499'
        >>> upstart_obj.daemon_proc['quit-plymouth']
        'stop/waiting'
        >>> upstart_obj.tty['/dev/tty4']['status']
        'stop/waiting'
        >>> sorted(upstart_obj.data)[0]
        'ck-log-system-restart stop/waiting'
    """

    def parse_content(self, content):
        self.data = []
        self.tty = {}
        self.daemon_proc = {}
        if (not content):
            raise SkipException("No Contents")
        for line in content:
            self.data.append(line)
            if 'dev/tty' in line:
                line_s = line.split()
                if len(line_s) > 1:
                    dev = line_s[1].replace('(', '').replace(')', '')
                    self.tty[dev] = {}
                    self.tty[dev]['status'] = line_s[2].replace(',', '')
                    if len(line_s) > 4 and 'process' in line:
                        self.tty[dev]['process'] = str(line_s[4])
            else:
                line_s = line.split()
                proc = line_s[0]
                status = line_s[1]
                self.daemon_proc[proc] = status

    def upstart_managed(self, daemon):
        """
        Arguments:
            daemon(str): The daemon name

        Returns:
            (str): This method returns the status of daemon service if it is managed by upstar else it will return `None`.
        """
        for line in self.data:
            if daemon in line:
                return line
        return None

    def daemon_status(self, daemon):
        """
        Arguments:
            daemon(str): The daemon name

        Returns:
            (str): This method will return the status of the process `start/running` or `stop/waiting` if it is managed by upstart else it will return `None`.
        """
        return self.daemon_proc.get(daemon, None)

    def dev_status(self, dev):
        """
        Arguments:
            dev(str): The tty device name

        Returns:
            (str): This method will return the status of the tty device `start/running` or `stop/waiting`, along with `process-ID` if it is managed by upstart else it will return `None`.
        """
        if dev and dev in self.tty.keys():
            return self.tty[dev].get('status', None)



from insights.parsers.cluster_conf import ClusterConf
from insights.tests import context_wrap

CLUSTER_CONF_INFO = """
<cluster name="mycluster" config_version="3">
   <clusternodes>
     <clusternode name="node-01.example.com" nodeid="1">
         <fence>
            <method name="APC">
                <device name="apc" port="1"/>
             </method>
            <method name="SAN">
                <device name="sanswitch1" port="12" action="on"/>
                <device name="sanswitch2" port="12" action="on"/>
            </method>
         </fence>
     </clusternode>
     <clusternode name="node-02.example.com" nodeid="2">
         <fence>
            <method name="APC">
              <device name="apc" port="2"/>
            </method>
            <method name="SAN">
                <device name="sanswitch1" port="12"/>
            </method>
         </fence>
     </clusternode>
    </clusternodes>
    <cman expected_votes="3"/>
    <fencedevices>
        <fencedevice agent="fence_imm" ipaddr="192.0.2.1" login="opmgr" name="fence1" passwd="***"/>
        <fencedevice agent="fence_imm" ipaddr="192.0.2.2" login="opmgr" name="fence2" passwd="***"/>
    </fencedevices>
   <rm>
    <resources>
       <lvm name="lvm" vg_name="shared_vg" lv_name="ha-lv"/>
       <fs name="FS" device="/dev/shared_vg/ha-lv" force_fsck="0" force_unmount="1" fsid="64050" fstype="ext4" mountpoint="/mnt" options="" self_fence="0"/>
    </resources>
   </rm>
</cluster>
"""


def test_cluster_conf():
    conf = ClusterConf(context_wrap(CLUSTER_CONF_INFO))
    assert any('clusternode' in line for line in conf.lines)



"""
PuppetCertExpireDate - command ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``
============================================================================================================

The PuppetCertExpireDate parser reads the output of
``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``.

Sample output of ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``::

    notAfter=Dec  4 07:04:05 2035 GMT

Examples::

    >>> type(date_info)
    <class 'insights.parsers.puppet_ca_cert_expire_date.PuppetCertExpireDate'>
    >>> date_info.expire_date
    datetime.datetime(2035, 12, 4, 7, 4, 5)

"""
from datetime import datetime

from insights import parser, CommandParser
from insights.specs import Specs
from insights.parsers import SkipException, ParseException


@parser(Specs.puppet_ca_cert_expire_date)
class PuppetCertExpireDate(CommandParser):
    """
    Read the ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``
    and set the date to property ``expire_date``.

    Attributes:
        expire_date (datetime): The date when the puppet ca cert will be expired
    """

    def parse_content(self, content):
        if len(content) == 1 and content[0].startswith('notAfter='):
            date_format = '%b %d %H:%M:%S %Y %Z'
            date_str = content[0].split('=', 1)[1]
            try:
                self.expire_date = datetime.strptime(date_str, date_format)
            except Exception:
                raise ParseException("Can not parse the date format")
        else:
            raise SkipException("Cannot get the puppet ca cert expire info")



import pytest
from insights.parsers.pam import PamConf, PamDConf, PamConfEntry
from insights.parsers import get_active_lines, pam
from insights.tests import context_wrap

import doctest


PAM_CONF_DATA = """
#%PAM-1.0
vsftpd      auth        required    pam_securetty.so
vsftpd      auth        requisite   pam_unix.so nullok
vsftpd      auth        sufficient  pam_nologin.so
vsftpd      account     optional    pam_unix.so
other       password    include     pam_cracklib.so retry=3 logging=verbose
other       password    required    pam_unix.so shadow nullok use_authtok
other       session     required    pam_unix.so
"""


def test_pam_conf():

    pam_conf = PamConf(context_wrap(PAM_CONF_DATA))
    assert len(pam_conf) == 7
    assert pam_conf[0].service == 'vsftpd'
    assert pam_conf[0].interface == 'auth'
    assert len(pam_conf[0].control_flags) == 1
    assert pam_conf[0].control_flags[0].flag == 'required'
    assert pam_conf[0].control_flags[0].value is None
    assert pam_conf[0].module_name == 'pam_securetty.so'
    assert pam_conf[0].module_args is None

    assert pam_conf[5].service == 'other'
    assert pam_conf[5].interface == 'password'
    assert len(pam_conf[5].control_flags) == 1
    assert pam_conf[5].control_flags[0].flag == 'required'
    assert pam_conf[5].control_flags[0].value is None
    assert pam_conf[5].module_name == 'pam_unix.so'
    assert pam_conf[5].module_args == "shadow nullok use_authtok"

    # Test of keyword search
    vsftpd = list(pam_conf.search(service='vsftpd'))
    assert len(vsftpd) == 4
    assert vsftpd[0] == pam_conf[0]
    assert vsftpd[1] == pam_conf[1]
    assert vsftpd[2] == pam_conf[2]
    assert vsftpd[3] == pam_conf[3]
    vsftpd_acct = list(pam_conf.search(service='vsftpd', interface='account'))
    assert len(vsftpd_acct) == 1
    assert vsftpd_acct[0] == pam_conf[3]

    # Test of individual entry repr
    assert repr(pam_conf[0]) == "<PamConfEntry for vsftpd: auth required pam_securetty.so>"
    assert repr(pam_conf[4]) == "<PamConfEntry for other: password include pam_cracklib.so retry=3 logging=verbose>"


PAMD_CONF_SSHD_DOCS = '''
#%PAM-1.0
auth       required     pam_sepermit.so
auth       substack     password-auth
auth       include      postlogin
# Used with polkit to reauthorize users in remote sessions
-auth      optional     pam_reauthorize.so prepare
account    required     pam_nologin.so
account    include      password-auth
password   include      password-auth
# pam_selinux.so close should be the first session rule
session    required     pam_selinux.so close
session    required     pam_loginuid.so
# pam_selinux.so open should only be followed by sessions to be executed in the user context
session    required     pam_selinux.so open env_params
session    required     pam_namespace.so
session    optional     pam_keyinit.so force revoke
session    include      password-auth
session    include      postlogin
# Used with polkit to reauthorize users in remote sessions
-session   optional     pam_reauthorize.so prepare
'''

PAMD_CONF_DATA = """
#%PAM-1.0
auth        required    pam_securetty.so
auth        requisite   pam_unix.so nullok
auth        sufficient  pam_nologin.so
auth        [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
account     optional    pam_unix.so
password    include     pam_cracklib.so retry=3 logging=verbose
password    required    pam_unix.so shadow nullok use_authtok
auth        required    pam_listfile.so \
                   onerr=succeed item=user sense=deny file=/etc/ftpusers
session     required    pam_unix.so
"""


def test_pamd_conf():

    pamd_conf = PamDConf(context_wrap(PAMD_CONF_DATA, path='etc/pamd.d/vsftpd'))
    assert len(pamd_conf) == 9
    assert pamd_conf[0].service == 'vsftpd'
    assert pamd_conf[0].interface == 'auth'
    assert len(pamd_conf[0].control_flags) == 1
    assert pamd_conf[0].control_flags[0].flag == 'required'
    assert pamd_conf[0].control_flags[0].value is None
    assert pamd_conf[0].module_name == 'pam_securetty.so'
    assert pamd_conf[0].module_args is None

    # auth        [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
    assert len(pamd_conf[3].control_flags) == 2
    assert pamd_conf[3].control_flags == [
        PamConfEntry.ControlFlag(flag='success', value='2'),
        PamConfEntry.ControlFlag(flag='default', value='ok')
    ]

    # auth        required    pam_listfile.so \
    #                   onerr=succeed item=user sense=deny file=/etc/ftpusers
    assert pamd_conf[7].interface == 'auth'
    assert len(pamd_conf[7].control_flags) == 1
    assert pamd_conf[7].control_flags[0].flag == 'required'
    assert pamd_conf[7].control_flags[0].value is None
    assert pamd_conf[7].module_name == 'pam_listfile.so'
    assert pamd_conf[7].module_args == 'onerr=succeed item=user sense=deny file=/etc/ftpusers'
    assert pamd_conf[7].module_args_dict == {
        'onerr': 'succeed', 'item': 'user', 'sense': 'deny',
        'file': '/etc/ftpusers'
    }

    # Test pam search for control flags
    default_set = pamd_conf.search(control_flags__contains='default')
    assert len(default_set) == 1
    assert default_set[0] == pamd_conf[3]
    verbose_set = pamd_conf.search(module_args__contains='verbose')
    assert len(verbose_set) == 1
    assert verbose_set[0] == pamd_conf[5]


PAMD_MISC = """
# Miscellaneous trickier lines
password  required pam_cracklib.so \
                       difok=3 minlen=15 dcredit= 2 ocredit=2
auth    requisite       pam_permit.so
auth    [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
password optional pam_exec.so seteuid /usr/bin/make -C /var/yp
-session optional pam_no_module.so ignore if module not found
"""


def test_pamd_misc_conf():

    # Miscellaneous complicated line handling tests
    misc = PamDConf(context_wrap(PAMD_MISC, path='etc/pamd.d/misc'))
    assert misc
    assert len(misc) == 5

    # password  required pam_cracklib.so \
    #                        difok=3 minlen=15 dcredit= 2 ocredit=2
    assert misc[0].service == 'misc'
    assert misc[0].interface == 'password'
    assert len(misc[0].control_flags) == 1
    assert misc[0].control_flags[0].flag == 'required'
    assert misc[0].control_flags[0].value is None
    assert misc[0].module_name == 'pam_cracklib.so'
    assert misc[0].module_args == 'difok=3 minlen=15 dcredit= 2 ocredit=2'

    # auth    requisite       pam_permit.so
    assert misc[1].service == 'misc'
    assert misc[1].interface == 'auth'
    assert len(misc[1].control_flags) == 1
    assert misc[1].control_flags[0].flag == 'requisite'
    assert misc[1].control_flags[0].value is None
    assert misc[1].module_name == 'pam_permit.so'
    assert misc[1].module_args is None

    # auth    [success=2 default=ok]  pam_debug.so auth=perm_denied cred=success
    assert misc[2].service == 'misc'
    assert misc[2].interface == 'auth'
    assert len(misc[2].control_flags) == 2
    assert misc[2].control_flags[0].flag == 'success'
    assert misc[2].control_flags[0].value == '2'
    assert misc[2].control_flags[1].flag == 'default'
    assert misc[2].control_flags[1].value == 'ok'
    assert misc[2].module_name == 'pam_debug.so'
    assert misc[2].module_args == 'auth=perm_denied cred=success'

    # password optional pam_exec.so seteuid /usr/bin/make -C /var/yp
    assert misc[3].service == 'misc'
    assert misc[3].interface == 'password'
    assert len(misc[3].control_flags) == 1
    assert misc[3].control_flags[0].flag == 'optional'
    assert misc[3].control_flags[0].value is None
    assert misc[3].module_name == 'pam_exec.so'
    assert misc[3].module_args == 'seteuid /usr/bin/make -C /var/yp'

    # -account optional pam_no_module.so ignore if module not found
    assert misc[4].service == 'misc'
    assert misc[4]._type_raw == '-session'
    assert misc[4].ignored_if_module_not_found
    assert misc[4].interface == 'session'
    assert len(misc[4].control_flags) == 1
    assert misc[4].control_flags[0].flag == 'optional'
    assert misc[4].control_flags[0].value is None
    assert misc[4].module_name == 'pam_no_module.so'
    assert misc[4].module_args == 'ignore if module not found'


PAMD_CONF_DATA_BAD_LINES = """
#%PAM-1.0
unrecognised_type
auth
auth badcontrol
auth [success=2 default] no_equals_in_group
auth required
"""


def test_bad_pamd_files():
    pamd_conf = PamDConf(context_wrap(PAMD_CONF_DATA_BAD_LINES, path='etc/pamd.d/bad'))
    assert pamd_conf
    assert len(pamd_conf) == 5
    for entry in pamd_conf:
        assert entry.service == 'bad'
        assert entry.interface is None
        assert entry.control_flags == []
        assert entry.module_name is None
        assert entry.module_args is None
        assert entry._errors == ["Cannot parse line '{f}' as a valid pam.d entry".format(f=entry._full_line)]


class PamDConfNoService(PamDConf):
    def parse_content(self, content):
        self.data = []
        for line in get_active_lines(content):
            self.data.append(PamConfEntry(line, pamd_conf=True))


def test_bad_pamd_subclass():
    with pytest.raises(ValueError) as exc:
        pamd_conf = PamDConfNoService(context_wrap(PAMD_CONF_DATA))
        assert pamd_conf is None
    assert 'Service name must be provided for pam.d conf file' in str(exc)


# Because tests are done at the module level, we have to put all the shared
# parser information in the one environment.  Fortunately this is normal.
def test_pam_doc_examples():
    env = {
        'PamDConf': PamDConf,
        'PamConfEntry': PamConfEntry,
        'pam_conf': PamConf(context_wrap(PAM_CONF_DATA, path='/etc/pam.conf')),
        'pamd_conf': PamDConf(context_wrap(PAMD_CONF_SSHD_DOCS, path='/etc/pam.d/sshd')),
    }
    failed, total = doctest.testmod(pam, globs=env)
    assert failed == 0



import json
import pytest
from insights.core import Parser

SIMPLE_DATA = json.dumps({"a": 1, "b": 2, "c": 3})

COMPLEX_DATA = json.dumps({
    "food": "processor",
    "mixing": "bowl",
    "cake": {
        "eggs": 2,
        "flour": 3,
        "sugar": 4,
        "chocolate": 10
    },
    "pie": {
        "peaches": 3,
        "sugar": 2,
        "dough": 1,
        "goo": 3
    }
})

FAKE_DATA = "This is some fake data"


class SimpleParser(Parser):

    def parse_content(self, content):
        self.data = json.loads(content)

    def __getitem__(self, i):
        return self.data[i]

    def ab(self):
        return self.data["a"] + self.data["b"]

    @property
    def bc(self):
        return self.data["b"] + self.data["c"]


class ComplexParser(SimpleParser):

    def cake(self):
        return SubParser(self.data["cake"])

    def pie(self):
        return SubParser(self.data["pie"])


class SubParser(object):

    def sugar(self):
        return self.data["sugar"]


class Context(object):
    def __init__(self, content):
        self.content = content
        self.path = ""
        self.relative_path = ""


def simple_parser(context):
    return SimpleParser(context)


def complex_parser(context):
    return ComplexParser(context)


def fake_parser(context):
    return context.content


for f in [simple_parser, complex_parser, fake_parser]:
    f.serializable_id = "#".join([f.__module__, f.__name__])


@pytest.fixture
def simple():
    return SimpleParser(Context(SIMPLE_DATA))


@pytest.fixture
def complex_():
    return ComplexParser(Context(COMPLEX_DATA))


def test_fn(simple):
    assert simple.ab() == 3


def test_computed(simple):
    assert simple.bc == 5


def test_getitem(complex_):
    assert complex_["food"] == "processor"
    assert complex_["mixing"] == "bowl"



import pytest
import doctest
from insights.parsers import ParseException, SkipException, mysqladmin
from insights.parsers.mysqladmin import MysqladminVars, MysqladminStatus
from insights.tests import context_wrap


OUTPUT_MYSQLADMIN_STATUS = """
Uptime: 1103965 Threads: 1820 Questions: 44778091 Slow queries: 0 Opens: 1919 Flush tables: 1 Open tables: 592 Queries per second avg: 40.561
""".strip()

BLANK_SAMPLE = """
""".strip()

BAD_INPUT_SAMPLE = """
Threads: 1820 Questions: 44778091 Slow queries: 0 Opens: 1919 Flush tables: 1 Open tables: 592 Queries per second avg: 40.561
""".strip()


def test_mysqladmin_status():
    parser_result = MysqladminStatus(context_wrap(OUTPUT_MYSQLADMIN_STATUS))
    mysqlstat = parser_result.status
    assert parser_result is not None
    assert mysqlstat['Threads'] == '1820'
    assert mysqlstat['Queries per second avg'] == '40.561'
    assert mysqlstat['Uptime'] == '1103965'
    assert mysqlstat['Opens'] == '1919'
    assert mysqlstat['Slow queries'] == '0'


def test_mysqlstat_blank_input():
    with pytest.raises(SkipException) as sc:
        MysqladminStatus(context_wrap(BLANK_SAMPLE))
    assert "Content is empty." in str(sc.value)


def test_mysqlstat_bad_input():
    with pytest.raises(ParseException) as exc:
        MysqladminStatus(context_wrap(BAD_INPUT_SAMPLE))
    assert "Unable to parse the output." in str(exc)


INPUT_NORMAL = """
+---------------------------------------------------+-------------------
| Variable_name                                     | Value            |
+---------------------------------------------------+------------------+
| aria_block_size                                   | 8192             |
| aria_checkpoint_interval                          | 30               |
| auto_increment_increment                          | 1                |
| auto_increment_offset                             | 1                |
| binlog_stmt_cache_size                            | 32768            |
| character_set_filesystem                          | binary           |
| datadir                                           | /var/lib/mysql/  |
| init_file                                         |                  |
| innodb_autoinc_lock_mode                          | 1                |
| version                                           | 5.5.56-MariaDB   |
| version_comment                                   | MariaDB Server   |
| version_compile_machine                           | x86_64           |
| version_compile_os                                | Linux            |
| wait_timeout                                      | 28800            |
+---------------------------------------------------+------------------+
""".strip()


def test_mysqladmin_vars():
    res = MysqladminVars(context_wrap(INPUT_NORMAL))
    d = res.data
    assert len(list(d)) == 14
    assert d['version_comment'] == 'MariaDB Server'
    assert d['datadir'] == '/var/lib/mysql/'
    assert d['auto_increment_increment'] == '1'
    assert d.get('abc') is None
    assert res.get('abc', '233') == '233'
    assert res.get('init_file') == ''
    assert res.get('wait_what') is None
    assert res.get('wait_timeout') == '28800'
    assert res.getint('wait_timeout') == 28800
    assert res.getint('version_compile_machine') is None
    assert res.get('binlog_stmt_cache_size', '666') == '32768'

    with pytest.raises(TypeError) as e_info:
        res.getint('binlog_stmt_cache_size', '666')
    assert "Default value should be int type." in str(e_info.value)


INPUT_EMPTY = """
+-------------------------------------------------+------------------+
| Variable_name                                   | Value            |
+-------------------------------------------------+------------------+
+-------------------------------------------------+------------------+
""".strip()

INPUT_FORAMT_WRONG = """
+-------------------------------------------------+------------------+
| Variable_name                                   | Value            |
+-------------------------------------------------+------------------+
| aria_block_size                                 | 1                |
| version_compile_machine                          x86_64           |
| version_compile_machine                         | x86_64           |x
| old_password********                                     ********
| version_compile_machine                           | x86_64           |
+-------------------------------------------------+------------------+
""".strip()


def test_empty_mysqladmin_var():
    with pytest.raises(SkipException) as e_info:
        MysqladminVars(context_wrap(""))
    assert "Empty content." in str(e_info.value)


def test_wrong_mysqladmin_var():
    with pytest.raises(ParseException) as e_info:
        MysqladminVars(context_wrap(INPUT_EMPTY))
    assert "Variable_name" in str(e_info.value)


INPUT_STILL_PARSABLE_1 = """
+-------------------------------------------------+------------------+
| Variable_name                                   | Value            |
+-------------------------------------------------+------------------+
| aria_block_size                                 | 1                |
| aria_checkpoint_interval                        | 30   |    23     |
+-------------------------------------------------+------------------+
""".strip()

INPUT_STILL_PARSABLE_2 = """
+---------------------------------------------------+-------------------
| Variable_name                                     | Value            |
+---------------------------------------------------+------------------+
| aria_block_size                                   | 8192             |
| aria_checkpoint_interval                          | 30               |
| ft_boolean_syntax                                 | + -><()~*:""&|   |
| version_compile_machine                           | x86_64           |
| version_compile_os                                | Linux            |
+---------------------------------------------------+------------------+
""".strip()

INPUT_FORAMT_WRONG = """
+-------------------------------------------------+------------------+
| Variable_name                                   | Value            |
+-------------------------------------------------+------------------+
| aria_block_size                                 | 1                |
| version_compile_machine                          x86_64           |
| version_compile_machine                         | x86_64           |x
| old_password********                                     ********
| version_compile_machine                           | x86_64           |
+-------------------------------------------------+------------------+
""".strip()


def test_mysqladmin_still_parsable():
    res = MysqladminVars(context_wrap(INPUT_STILL_PARSABLE_1))
    d = res.data
    assert d['aria_checkpoint_interval'] == '30   |    23'

    res = MysqladminVars(context_wrap(INPUT_STILL_PARSABLE_2))
    d = res.data
    assert d['ft_boolean_syntax'] == '+ -><()~*:""&|'

    res = MysqladminVars(context_wrap(INPUT_FORAMT_WRONG))
    assert len(res.bad_lines) == 3
    assert res.bad_lines[1] == '| version_compile_machine                         | x86_64           |x'


def test_doc():
    env = {
            'MysqladminStatus': MysqladminStatus,
            'result': MysqladminStatus(context_wrap(OUTPUT_MYSQLADMIN_STATUS, path='/bin/mysqladmin status')),
            'MysqladminVars': MysqladminVars,
            'output': MysqladminVars(context_wrap(INPUT_NORMAL, '/bin/mysqladmin variables')),
    }
    failed, total = doctest.testmod(mysqladmin, globs=env)
    assert failed == 0



from insights.parsers.sysconfig import DockerSysconfigStorage
from insights.tests import context_wrap


DOCKER_CONFIG_STORAGE = """
DOCKER_STORAGE_OPTIONS="--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true"
""".strip()


def test_sysconfig_docker_content():
    context = context_wrap(DOCKER_CONFIG_STORAGE, 'etc/sysconfig/docker-storage')
    sysconf = DockerSysconfigStorage(context)

    assert sorted(sysconf.keys()) == sorted(['DOCKER_STORAGE_OPTIONS'])
    assert 'DOCKER_STORAGE_OPTIONS' in sysconf
    assert sysconf['DOCKER_STORAGE_OPTIONS'] == "--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true"
    assert sysconf.storage_options == "--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true"



"""
LdLibraryPath - LD_LIBRARY_PATH of PIDs
=======================================

Parser for parsing the environment variable LD_LIBRARY_PATH of each PID.

"""

from collections import namedtuple
from insights import parser, Parser
from insights.parsers import SkipException, ParseException
from insights.specs import Specs

LdLibraryPath = namedtuple('LdLibraryPath', ('pid', 'path', 'raw'))
"""namedtuple: Type for storing the LdLibraryPath of PID"""


@parser(Specs.ld_library_path_of_pid)
class PidLdLibraryPath(Parser, list):
    """
    Base class for parsing the ``LD_LIBRARY_PATH`` variable of each PID of the
    system into a list.

    Typical content looks like::

        105901 /usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib
        105902 /usr/sap/RH1/SYS/exe/uc/linuxx86_64:/usr/sap/RH1/SYS/exe/run

    Examples:
        >>> len(ld_lib_path)
        2
        >>> isinstance(ld_lib_path[0].path, list)
        True
        >>> len(ld_lib_path[0].path)
        3
        >>> '/sapdb/clients/RH1/lib' in ld_lib_path[0].path
        True
        >>> ld_lib_path[0].pid
        '105901'

    Raises:
        SkipException: When the output is empty or nothing needs to parse.
        ParseException: When the line cannot be parsed.

    """

    def parse_content(self, content):
        if not content:
            raise SkipException

        llds = []
        for line in content:
            pid, _, raw = [s.strip() for s in line.partition(' ')]
            paths = raw
            if not pid.isdigit():
                raise ParseException('Incorrect line: {0}'.format(line))
            if raw and raw[0] == raw[-1] and raw[0] in ('\'', '"'):
                paths = raw[1:-1]
            paths = paths.split(':')
            llds.append(LdLibraryPath(pid, paths, raw))

        if not llds:
            raise SkipException("LD_LIBRARY_PATH not set.")

        self.extend(llds)



'''
VirtWhat - Command ``virt-what``
================================

Parses the output of the ``virt-what`` command to check if the host is running
in a virtual machine.

Sample input::
    kvm

Examples:
    >>> vw = shared[VirtWhat]
    >>> vw.is_virtual
    True
    >>> vw.is_physical
    False
    >>> vw.generic
    'kvm'
    >>> 'aws' in vw
    False

Note:
    For ``virt-what-1.13-8`` or older on RHEL7, the command fails when running
    without an environment (or very restricted environment), and reports below
    error::

        virt-what: virt-what-cpuid-helper program not found in $PATH

'''
from .. import parser, CommandParser
from insights.specs import Specs

BAREMETAL = 'baremetal'


@parser(Specs.virt_what)
class VirtWhat(CommandParser):
    """
    Class for parsing ``virt-what`` command.

    Attributes:
        generic (str): The type of the virtual machine. 'baremetal' if physical machine.
        errors (list): List of the error information if any error occurs.
        specifics (list): List of the specific information if the command outputs.

    """

    def __init__(self, *args, **kwargs):
        self.generic = ''
        self.errors = []
        self.specifics = []
        super(VirtWhat, self).__init__(*args, **kwargs)

    def __contains__(self, name):
        """bool: Is this ``name`` found in the specific list?"""
        return name in [self.generic] + self.specifics

    @property
    def is_virtual(self):
        """
        bool: Is the host running in a virtual machine? None when something is
              wrong.
        """
        if not self.errors:
            return self.generic != BAREMETAL

    @property
    def is_physical(self):
        """
        bool: Is the host running in a physical machine? None when something is
              wrong.
        """
        if not self.errors:
            return self.generic == BAREMETAL

    def parse_content(self, content):
        if content:
            if len(content[0].split()) > 1:
                self.errors = [line for line in content]
            else:
                self.generic = content[0]
                self.specifics = [line for line in content[1:]]
        else:
            self.generic = BAREMETAL



"""
RdmaConfig - file ``/etc/rdma/rdma.conf``
=========================================
"""
from .. import get_active_lines, Parser, parser, LegacyItemAccess
from insights.parsers import split_kv_pairs
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.rdma_conf)
class RdmaConfig(Parser, LegacyItemAccess):
    """
    This class will parse the output of file ``/etc/rdma/rdma.conf``.

    The rdma service reads /etc/rdma/rdma.conf file to find out which
    kernel-level and user-level RDMA protocols the administrator
    wants to be loaded by default.

    Attributes:
        data (dict): Dictionary of keys with values in dict.

    Sample configuration file::

        IPOIB_LOAD=yes
        # Load SRP (SCSI Remote Protocol initiator support) module
        SRP_LOAD=yes
        # Load SRPT (SCSI Remote Protocol target support) module
        SRPT_LOAD=yes
        # Load iSER (iSCSI over RDMA initiator support) module
        ISER_LOAD=yes
        # Load iSERT (iSCSI over RDMA target support) module
        ISERT_LOAD=yes
        # Load RDS (Reliable Datagram Service) network protocol
        RDS_LOAD=no
        # Load NFSoRDMA client transport module
        XPRTRDMA_LOAD=yes
        # Load NFSoRDMA server transport module
        SVCRDMA_LOAD=no
        # Load Tech Preview device driver modules
        TECH_PREVIEW_LOAD=no
        # Should we modify the system mtrr registers?  We may need to do this if you
        # get messages from the ib_ipath driver saying that it couldn't enable
        # write combining for the PIO buffs on the card.
        #
        # Note: recent kernels should do this for us, but in case they don't, we'll
        # leave this option
        FIXUP_MTRR_REGS=no

    Examples:
        >>> rdma_conf['IPOIB_LOAD']
        'yes'
        >>> rdma_conf["SRP_LOAD"]
        'yes'
        >>> rdma_conf["SVCRDMA_LOAD"]
        'no'
    """

    def parse_content(self, content):
        _content = get_active_lines(content)
        if not _content:
            raise SkipException("Empty content.")

        self.data = split_kv_pairs(_content)



import doctest
import pytest
from insights.parsers import docker_list
from insights.parsers import SkipException
from insights.tests import context_wrap


DOCKER_LIST_IMAGES = """
REPOSITORY                           TAG                 DIGEST              IMAGE ID                                                           CREATED             VIRTUAL SIZE
rhel6_vsftpd                         latest              <none>              412b684338a1178f0e5ad68a5fd00df01a10a18495959398b2cf92c2033d3d02   37 minutes ago      459.5 MB
rhel7_imagemagick                    latest              <none>              882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2   4 days ago          785.4 MB
rhel6_nss-softokn                    latest              <none>              dd87dad2c7841a19263ae2dc96d32c501ee84a92f56aed75bb67f57efe4e48b5   5 days ago          449.7 MB
<none>                               <none>              <none>              34c167d900afb820ecab622a214ce3207af80ec755c0dcb6165b425087ddbc3a   5 days ago          205.3 MB
""".strip()

DOCKER_LIST_CONTAINERS = """
CONTAINER ID                                                       IMAGE                                                              COMMAND                                            CREATED             STATUS                      PORTS                  NAMES               SIZE
03e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216   rhel7_httpd                                                        "/usr/sbin/httpd -DFOREGROUND"                     45 seconds ago      Up 37 seconds               0.0.0.0:8080->80/tcp   angry_saha          796 B (virtual 669.2 MB)
95516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102   bd8638c869ea40a9269d87e9af6741574562af9ee013e03ac2745fb5f59e2478   "/bin/sh -c 'yum install -y vsftpd-2.2.2-6.el6'"   18 hours ago        Exited (137) 18 hours ago                          tender_rosalind     4.751 MB (virtual 200.4 MB)
""".strip()

DOCKER_LIST_IMAGES_NO_DATA = """
REPOSITORY                           TAG                 DIGEST              IMAGE ID                                                           CREATED             VIRTUAL SIZE
"""


def test_docker_list_images():
    result = docker_list.DockerListImages(context_wrap(DOCKER_LIST_IMAGES))
    # All rows get read:
    assert len(result.rows) == 4
    # Rows with data are as normal
    assert result.rows[0].get("REPOSITORY") == "rhel6_vsftpd"
    assert result.rows[0].get("TAG") == "latest"
    assert result.rows[0].get("DIGEST") == "<none>"
    assert result.rows[0].get("IMAGE ID") == '412b684338a1178f0e5ad68a5fd00df01a10a18495959398b2cf92c2033d3d02'
    assert result.rows[0].get("CREATED") == "37 minutes ago"
    assert result.rows[0].get("VIRTUAL SIZE") == "459.5 MB"
    # Rows with <none> still get processed.
    assert result.rows[3].get("REPOSITORY") == "<none>"
    assert result.rows[1].get("TAG") == "latest"
    assert result.rows[3].get("IMAGE ID") == '34c167d900afb820ecab622a214ce3207af80ec755c0dcb6165b425087ddbc3a'
    assert result.rows[2].get("REPOSITORY") == "rhel6_nss-softokn"
    assert result.rows[2].get("TAG") == "latest"
    assert result.rows[2].get("IMAGE ID") == 'dd87dad2c7841a19263ae2dc96d32c501ee84a92f56aed75bb67f57efe4e48b5'

    assert result.data['rhel6_vsftpd']['CREATED'] == '37 minutes ago'
    # Same data in both accessors
    assert result.data['rhel6_vsftpd'] == result.rows[0]
    # Can't list repositories if they don't have a repository name
    assert '<none>' not in result.data


def test_docker_list_containers():
    result = docker_list.DockerListContainers(context_wrap(DOCKER_LIST_CONTAINERS))
    assert len(result.rows) == 2
    assert result.rows[0].get("CONTAINER ID") == "03e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216"
    assert result.rows[0].get("COMMAND") == '"/usr/sbin/httpd -DFOREGROUND"'
    assert result.rows[0].get("SIZE") == "796 B (virtual 669.2 MB)"
    assert result.rows[0].get("CREATED") == "45 seconds ago"
    assert result.rows[0].get("PORTS") == "0.0.0.0:8080->80/tcp"
    assert result.rows[1].get("CONTAINER ID") == "95516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102"
    assert result.rows[1].get("COMMAND") == '"/bin/sh -c \'yum install -y vsftpd-2.2.2-6.el6\'"'
    assert result.rows[1]['STATUS'] == 'Exited (137) 18 hours ago'
    assert result.rows[1].get("PORTS") == ''

    assert sorted(result.data.keys()) == sorted(['angry_saha', 'tender_rosalind'])
    assert result.data['angry_saha'] == result.rows[0]
    assert result.data['tender_rosalind'] == result.rows[1]


def test_docker_list_images_no_data():
    with pytest.raises(SkipException) as ex:
        docker_list.DockerListImages(context_wrap(DOCKER_LIST_IMAGES_NO_DATA))
    assert 'No data.' in str(ex)


def test_undefined_key_field():
    with pytest.raises(NotImplementedError):
        assert docker_list.DockerList(context_wrap(DOCKER_LIST_CONTAINERS)).key_field is None


def test_documentation():
    failed_count, tests = doctest.testmod(
        docker_list,
        globs={'images': docker_list.DockerListImages(context_wrap(DOCKER_LIST_IMAGES)),
               'containers': docker_list.DockerListContainers(context_wrap(DOCKER_LIST_CONTAINERS))}
    )
    assert failed_count == 0



import doctest
from insights.parsers import ls_var_opt_mssql_log
from insights.parsers.ls_var_opt_mssql_log import LsVarOptMssqlLog
from insights.tests import context_wrap

MSSQL_LOG = """
total 6322200
drwxr-xr-x.  3 mssql mssql        256 Jun  7 10:07 .
drwxr-xr-x. 71 root  root        4096 Jun 22 10:35 ..
drwxr-xr-x.  2 mssql mssql         65 Jul 10 09:33 journal
-rw-------.  1 mssql mssql   67108864 Jul 10 09:32 local.0
"""


def test_ls_var_opt_mssql_log():
    log = LsVarOptMssqlLog(context_wrap(MSSQL_LOG, path="insights_commands/ls_-la_.var.opt.mssql.log"))
    assert log.dirs_of('/var/opt/mssql/log') == ['.', '..', 'journal']
    journal = log.dir_entry('/var/opt/mssql/log', 'journal')
    print(journal)
    assert journal == {
            'type': 'd', 'perms': 'rwxr-xr-x.', 'links': 2, 'owner': 'mssql',
            'group': 'mssql', 'size': 65, 'date': 'Jul 10 09:33',
            'raw_entry': 'drwxr-xr-x.  2 mssql mssql         65 Jul 10 09:33 journal',
            'name': 'journal', 'dir': '/var/opt/mssql/log'}


def test_ls_var_opt_mssql_log_examples():
    env = {
        'ls_mssql_log': LsVarOptMssqlLog(context_wrap(MSSQL_LOG, path="insights_commands/ls_-la_.var.opt.mssql.log")),
    }
    failed, total = doctest.testmod(ls_var_opt_mssql_log, globs=env)
    assert failed == 0



"""
SELinux
=======

Combiner for more complex handling of SELinux being disabled by any means
available to the users. It uses results of
:class:`insights.parsers.sestatus.SEStatus`, and
:class:`insights.parsers.selinux_config.SelinuxConfig` parsers and
:class:`insights.combiners.grub_conf.GrubConf` combiner.

It contains a dictionary ``problems`` in which it stores detected problems with
keys as follows and values are parsed lines with detected problem:

* ``sestatus_disabled`` - SELinux is disabled on runtime.
* ``sestatus_not_enforcing`` - SELinux is not in enforcing mode.
* ``grub_disabled`` - SELinux is set in Grub to be disabled.
* ``grub_not_enforcing`` - SELinux is set in Grub to not be in enforcing mode.
* ``selinux_conf_disabled`` - SELinux is set in configuration file to be disabled.
* ``sestatus_not_enforcing`` - SELinux is set in configuration file to not be in enforcing mode.

Examples:
    >>> selinux = shared[SELinux]
    >>> selinux.ok()
    False
    >>> selinux.problems
    {'grub_disabled': ['/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 ro root= ...'],
     'selinux_conf_disabled': 'disabled',
     'sestatus_not_enforcing': 'permissive'}
"""

from ..core.plugins import combiner
from ..parsers.sestatus import SEStatus
from ..combiners.grub_conf import GrubConf
from ..parsers.selinux_config import SelinuxConfig

GRUB_DISABLED = 'grub_disabled'
GRUB_NOT_ENFORCING = 'grub_not_enforcing'
RUNTIME_DISABLED = 'sestatus_disabled'
RUNTIME_NOT_ENFORCING = 'sestatus_not_enforcing'
BOOT_DISABLED = 'selinux_conf_disabled'
BOOT_NOT_ENFORCING = 'selinux_conf_not_enforcing'


@combiner(SEStatus, SelinuxConfig, optional=[GrubConf])
class SELinux(object):
    """
    A combiner for detecting that SELinux is enabled and running and also enabled at boot time.
    """
    def __init__(self, se_status, selinux_config, grub_conf):
        self.problems = {}
        self.sestatus = se_status
        self.selinux_config = selinux_config
        self.grub_config = grub_conf.grub if grub_conf else None

        self._check_sestatus()
        self._check_boot_config()
        self._check_grub_config()

    def _check_sestatus(self):
        """
        Check runtime SELinux configuration from sestatus output.

        Values of output from sestatus command are always lowercase.
        """
        if self.sestatus.data['selinux_status'] != 'enabled':
            self.problems[RUNTIME_DISABLED] = self.sestatus.data['selinux_status']
        elif self.sestatus.data['current_mode'] != 'enforcing':
            self.problems[RUNTIME_NOT_ENFORCING] = self.sestatus.data['current_mode']

    def _check_boot_config(self):
        """
        Check that SELinux is not disabled in /etc/sysconfig/selinux.

        This file determines the boot configuration for SELinux.
        """
        opt_value = self.selinux_config.data.get('SELINUX')
        if opt_value is None:
            self.problems[BOOT_NOT_ENFORCING] = 'Missing in config (Permissive by default)'
        elif opt_value == 'disabled':
            self.problems[BOOT_DISABLED] = opt_value
        elif opt_value != 'enforcing':
            self.problems[BOOT_NOT_ENFORCING] = opt_value

    def _check_grub_config(self):
        """
        Check grub and grub 2 for kernel boot options if selinux settings is not overriden.

        Experiments confirmed that only lowercase is accepted in grub configuration.

        grub is in rhel-6 and the boot line looks usually like
            kernel /boot/vmlinuz-2.4.20-selinux-2003040709 ro root=/dev/hda1 nousb selinux=0
        grub 2 is in rhel-7 and rhel-8, the boot line of grub 2 in rhel-7 looks usually like
            linux16 /vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
        the boot line of grub 2 in rhel-8 looks usually like
            options root=/dev/mapper/rhel_vm37--146-root ro crashkernel=auto resume=/dev/mapper/rhel_vm37--146-swap rd.lvm.lv=rhel_vm37-146/root rd.lvm.lv=rhel_vm37-146/swap $tuned_params
        """

        conf = self.grub_config.boot_entries if self.grub_config is not None else []
        se_dis = [e.cmdline for e in conf if 'selinux=0' in e.cmdline]
        if se_dis:
            self.problems[GRUB_DISABLED] = se_dis
        se_noe = [e.cmdline for e in conf if 'enforcing=0' in e.cmdline]
        if se_noe:
            self.problems[GRUB_NOT_ENFORCING] = se_noe

    def ok(self):
        """
        Checks if there are any problems with SELinux configuration.

        Returns
            bool: True if SELinux is enabled and functional, false otherwise.
        """
        return not bool(self.problems)



"""engine_config - command ``engine-config --all``
==================================================

This module provides access to ovirt-engine configuration parameters
by parsing output of command ``engine-config --all``.
"""
from collections import namedtuple
from operator import itemgetter

from .. import parser, get_active_lines, CommandParser
from insights.specs import Specs


@parser(Specs.engine_config_all)
class EngineConfigAll(CommandParser):
    """Parsing output of command ``engine-config --all``

    The parser tries its best to get `value` & `version` for specified
    `keyword`. At the moment it works well for output which has
    `keyword`, `value` & `version` in a single line. It ignores
    keywords where is fails. It skip(rather fails) the `keyword`
    having multi-line output.

    Typical output of ``engine-config --all`` command is::

        MaxRerunVmOnVdsCount: 3 version: general
        MaxStorageVdsTimeoutCheckSec: 30 version: general
        ClusterRequiredRngSourcesDefault:  version: 3.6
        ClusterRequiredRngSourcesDefault:  version: 4.0
        ClusterRequiredRngSourcesDefault:  version: 4.1
        HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 3.6
        HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 4.0
        HotUnplugCpuSupported: {"x86":"true","ppc":"true"} version: 4.1


    Examples:

        >>> from insights.parsers.engine_config import EngineConfigAll
        >>> from insights.tests import context_wrap
        >>> output = EngineConfigAll(context_wrap(OUTPUT))}
        >>> 'MaxRerunVmOnVdsCount' in output
        True
        >>> output['MaxRerunVmOnVdsCount']
        ['3']
        >>> output.get('MaxRerunVmOnVdsCount')
        ['3']
        >>> output['HotUnplugCpuSupported']
        ['{"x86":"false","ppc":"false"}', '{"x86":"false","ppc":"false"}', '{"x86":"true","ppc":"true"}']
        >>> output['ClusterRequiredRngSourcesDefault']
        []
        >>> output.head('HotUnplugCpuSupported')
        '{"x86":"false","ppc":"false"}'
        >>> output.last('HotUnplugCpuSupported')
        '{"x86":"true","ppc":"true"}'
        >>> output.get_version('HotUnplugCpuSupported')
        ['3.6', '4.0', '4.1']
        >>> 'IDoNotExit' in output
        False
        >>> output['IDoNotExit']
        []
        >>> output.get('IDoNotExit')
        []
        >>> output.get_version('IDoNotExit')
        []
        >>> output.head('IDoNotExist)
        >>> output.last('IDoNotExist)

    Attributes:
        fields (list): List of `KeyValue` namedtupules for
                       each line in the configuration file.

        keywords (set): Set of keywords present in the configuration
                        file, each keyword has been converted to lowercase.
    """
    keyvalue = namedtuple('KeyValue',
                          ['keyword', 'value', 'version', 'kw_lower'])
    """namedtuple: Represent name value pair as a namedtuple with case."""

    def parse_content(self, content):
        """Parse each active line for keyword, values & version.

        Args:

            content (list): Output of command ``engine-config --all``.
        """
        self.fields = []
        for line in get_active_lines(content):
            try:
                key, val, ver = itemgetter(0, 1, -1)(line.split(' '))
                self.fields.append(self.keyvalue(key.strip(':'), val, ver, key.strip(':').lower()))  # noqa
            except:
                # TODO: Log an exception.
                pass
        self.keywords = set([kw.kw_lower for kw in self.fields])

    def __contains__(self, keyword):
        return keyword.lower() in self.keywords

    def __iter__(self):
        return iter(self.fields)

    def __getitem__(self, keyword):
        return self.get(keyword)

    def get(self, keyword):
        """A get value for keyword specified. A "dictionary like" method.

        Example:

            >>> output.get('MaxStorageVdsTimeoutCheckSec')
            ['30']

        Args:

            keyword (str): A key. For ex. `HotUnplugCpuSupported`.

        Returns:

            list: Values associated with a keyword. Returns an empty
            list if, all the values are empty or `keyword` does not
            exist.
        """
        kw = keyword.lower()
        if kw in self.keywords:
            return [kv.value for kv in self.fields if kv.kw_lower == kw if kv.value]
        return []

    def head(self, keyword):
        """Get first element from values(list).

        Example:

            >>> output['HotUnplugCpuSupported']
            ['{"x86":"false","ppc":"false"}', '{"x86":"false","ppc":"false"}', '{"x86":"true","ppc":"true"}']
            >>> output.head('HotUnplugCpuSupported')
            '{"x86":"false","ppc":"false"}'

        Args:

            keyword (str): A key. For ex. `HotUnplugCpuSupported`.

        Returns:

            str: First element from values(list) associated with a keyword else None
        """
        values = self.__getitem__(keyword)
        if values:
            return values[0]

    def last(self, keyword):
        """Get last element from values(list).

        Example:

            >>> output['HotUnplugCpuSupported']
            ['{"x86":"false","ppc":"false"}', '{"x86":"false","ppc":"false"}', '{"x86":"true","ppc":"true"}']
            >>> output.last('HotUnplugCpuSupported')
            '{"x86":"true","ppc":"true"}'

        Args:

            keyword (str): A key. For ex. `HotUnplugCpuSupported`.

        Returns:

            str: Last element from values(list) associated with a keyword else None.
        """
        values = self.__getitem__(keyword)
        if values:
            return values[-1]

    def get_version(self, keyword):
        """Get versions associated with a key.

        Typical output is ``engine-config --all`` command::

            MaxStorageVdsTimeoutCheckSec: 30 version: general
            HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 3.6
            HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 4.0
            HotUnplugCpuSupported: {"x86":"true","ppc":"true"} version: 4.1

        Examples:

            >>> output.get_version('MaxStorageVdsTimeoutCheckSec')
            ['general']
            >>> output.get_version('HotUnplugCpuSupported')
            ['3.6', '4.0', '4.1']

        Args:

            keyword (str): A key. For ex. `HotUnplugCpuSupported`.

        Returns:

            list: Versions associated with a keyword. Returns an empty
            list if, all the versions are empty or `keyword` does not
            exist.
        """
        kw = keyword.lower()
        if kw in self.keywords:
            return [kv.version for kv in self.fields if kv.kw_lower == kw if kv.version]
        return []



from insights.parsers.engine_log import EngineLog
from insights.tests import context_wrap

from datetime import datetime

ENGINE_LOG = """
2016-05-18 13:21:21,115 INFO [org.ovirt.engine.core.bll.scheduling.policyunits.EvenGuestDistributionBalancePolicyUnit] (DefaultQuartzScheduler_Worker-95) [5bc194fa] There is no host with more than 8 running guests, no balancing is needed
2016-05-18 14:00:51,272 INFO [org.ovirt.engine.core.vdsbroker.VdsUpdateRunTimeInfo] (DefaultQuartzScheduler_Worker-95) [5bc194fa] VM ADLG8201 ab289661-bbaa-4d27-a67a-ad20626f60f0 moved from PoweringUp --> Paused
2016-05-18 14:00:51,318 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.
2016-05-18 14:00:51,317 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.
"""

matched_lines = ['2016-05-18 14:00:51,318 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.',
                '2016-05-18 14:00:51,317 ERROR [org.ovirt.engine.core.dal.dbbroker.auditloghandling.AuditLogDirector] (DefaultQuartzScheduler_Worker-95) [5bc194fa] Correlation ID: null, Call Stack: null, Custom Event ID: -1, Message: VM ADLG8201 has paused due to storage I/O problem.']


def test_engine_log():
    engine_log_obj = EngineLog(context_wrap(ENGINE_LOG))
    assert "storage I/O problem." in engine_log_obj
    assert matched_lines == [i['raw_message'] for i in engine_log_obj.get('has paused due to storage I/O problem')]
    assert len(list(engine_log_obj.get_after(datetime(2016, 5, 18, 14, 0, 0)))) == 3



from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs


@parser(Specs.sestatus)
class SEStatus(LegacyItemAccess, CommandParser):
    """Class to parse the ``sestatus -b`` command

    Attributes:
        data (dict): A dict likes
        {
            "loaded_policy_name": "targeted",
            "policy_booleans": {
                "antivirus_use_jit": False,
                "abrt_anon_write": False,
                "abrt_upload_watch_anon_write": True,
                "antivirus_can_scan_system": False,
                "abrt_handle_event": False,
                "auditadm_exec_content": True,
            },
            "mode_from_config_file": "enforcing",
            "current_mode": "enforcing",
            "policy_mls_status": "enabled",
            "policy_deny_unknown_status": "allowed",
            "max_kernel_policy_version": "30"
        }

    ---Sample---
        Loaded policy name:             targeted
        Current mode:                   enforcing
        Mode from config file:          enforcing
        Policy MLS status:              enabled
        Policy deny_unknown status:     allowed
        Max kernel policy version:      30

        Policy booleans:
        abrt_anon_write                             off
        abrt_handle_event                           off
        abrt_upload_watch_anon_write                on
        antivirus_can_scan_system                   off
        antivirus_use_jit                           off
        auditadm_exec_content                       on
        ...
    """

    def parse_content(self, content):
        # Default to disabled if not found
        sestatus_info = {
            'loaded_policy_name': None,
            'current_mode': 'disabled',
            'mode_from_config_file': 'disabled',
            'policy_mls_status': 'disabled',
            'policy_deny_unknown_status': 'disabled',
            'max_kernel_policy_version': None,
            'policy_booleans': {},
        }

        for line in content:
            if ":" in line:
                if 'Policy booleans' in line:
                    pass
                else:
                    key, val = [s.strip() for s in line.split(":", 1)]
                    sestatus_info[key.lower().replace(" ", "_")] = val
            else:
                if line.strip():
                    key, val = line.split()
                    #  convert 'on' and 'off' strings to actual boolean values
                    sestatus_info['policy_booleans'][key] = val == 'on'

        # When SELinux is disabled, sestatus has simply 'SELinux status: disabled'
        # in its output.  But 'SELinux status' is not included in the output
        # when SELinux is enabled.  So we include it as a nicety.
        if sestatus_info['current_mode'] != 'disabled' and 'selinux_status' not in sestatus_info:
            sestatus_info['selinux_status'] = sestatus_info['current_mode']

        self.data = sestatus_info



"""
getsebool - command ``/usr/sbin/getsebool -a``
==============================================

This parser returns the output of the ``getsebool``
command.

Sample ``getsebool -a`` output::

    webadm_manage_user_files --> off
    webadm_read_user_files --> off
    wine_mmap_zero_ignore --> off
    xdm_bind_vnc_tcp_port --> off
    ssh_keysign --> off

Examples:

    >>> "webadm_manage_user_files" in getsebool
    True
    >>> "tmpreaper_use_nfs" in getsebool
    False
    >>> getsebool['ssh_keysign']
    'off'
"""

from .. import parser, LegacyItemAccess, CommandParser
from . import SkipException
from insights.specs import Specs


@parser(Specs.getsebool)
class Getsebool(LegacyItemAccess, CommandParser):
    """
    The output of "getsebool" command is like following:

        tmpreaper_use_nfs --> off
        tmpreaper_use_samba --> off

    So we can return the value like {"tmpreaper_use_nfs":"off", "tmpreaper_use_samba":"off"}

    Raises:
        SkipException: When SELinux is not enabled.
    """

    def parse_content(self, content):
        if content and 'selinux is disabled' in content[0].lower():
            raise SkipException('SELinux is disabled')

        self.data = {}
        for line in content:
            key, value = line.split("-->")
            self.data[key.strip()] = value.strip()



"""
DnsmasqConfAll - files ``/etc/dnsmasq.conf`` and ``/etc/dnsmasq.d/*.conf``
==========================================================================

Combiner for dnsmasq comfiguration files.

The man page http://www.thekelleys.org.uk/dnsmasq/docs/dnsmasq-man.html
states that dnsmasq reads /etc/dnsmasq.conf at startup, if it
exists. If conf-dir in dnsmasq.conf specified, then reads files in the
given directory by conf-dir option.

Configurations from *.conf files in the directory /etc/dnsmasq.d/ applied
when conf-dir set to:

conf-dir=/etc/dnsmasq.d
conf-dir=/etc/dnsmasq.d/,*.conf

"""
import os
import operator
from fnmatch import fnmatch

from insights.core.plugins import combiner
from insights.core import ConfigCombiner
from insights.parsers.dnsmasq_config import DnsmasqConf
from insights.parsr.query import eq


@combiner(DnsmasqConf)
class DnsmasqConfTree(ConfigCombiner):
    def __init__(self, confs):
        include = eq("conf-dir")
        main_file = "dnsmasq.conf"
        super(DnsmasqConfTree, self).__init__(confs, main_file, include)

    def find_matches(self, confs, pattern):
        results = []
        if ',' in pattern:
            pattern_split = pattern.split(',')
            # Include all the files in a directory except those ending in .conf
            # conf-dir=/etc/dnsmasq.d,.conf
            if ".conf" in pattern_split[1:]:
                return results
            pattern = pattern_split[0]

        # conf-dir=/etc/dnsmasq.d/
        if os.path.dirname(pattern):
            pattern = os.path.join(pattern, '*')

        for c in confs:
            if fnmatch(c.file_path, pattern):
                results.append(c)

        return sorted(results, key=operator.attrgetter("file_name"))

    @property
    def conf_path(self):
        return "/etc/dnsmasq.d"



import doctest

from insights.parsers import ls_run_systemd_generator
from insights.parsers.ls_run_systemd_generator import LsRunSystemdGenerator
from insights.tests import context_wrap

LS_RUN_SYSTEMD_GENERATOR = """
total 28
drwxr-xr-x.  6 0 0 260 Aug  5 07:35 .
drwxr-xr-x. 18 0 0 440 Aug  5 07:35 ..
-rw-r--r--.  1 0 0 254 Aug  5 07:35 boot.mount
-rw-r--r--.  1 0 0 259 Aug  5 07:35 boot\x2dfake.mount
-rw-r--r--.  1 0 0 176 Aug  5 07:35 dev-mapper-rhel\x2dswap.swap
drwxr-xr-x.  2 0 0 100 Aug  5 07:35 local-fs.target.requires
-rw-r--r--.  1 0 0 217 Aug  5 07:35 -.mount
drwxr-xr-x.  2 0 0  60 Aug  5 07:35 nfs-server.service.d
drwxr-xr-x.  2 0 0 100 Aug  5 07:35 remote-fs.target.requires
-rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt_nfs3.mount
-rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt\x2dnfs1.mount
-rw-r--r--.  1 0 0 261 Aug  5 07:35 root-mnt\x2dnfs2.mount
drwxr-xr-x.  2 0 0  60 Aug  5 07:35 swap.target.requires
""".strip()

PATH_OF_LS_RUN_SYSTEMD_GENERATOR = "insights_commands/ls_-lan_.run.systemd.generator"


def test_ls_run_systemd_generator():
    ls = LsRunSystemdGenerator(context_wrap(LS_RUN_SYSTEMD_GENERATOR,
                               path=PATH_OF_LS_RUN_SYSTEMD_GENERATOR))
    dir_path = "/run/systemd/generator"
    assert dir_path in ls
    assert len(ls.files_of(dir_path)) == 7
    assert ls.files_of(dir_path) == ['boot.mount', 'boot-fake.mount', 'dev-mapper-rhel-swap.swap',
                                '-.mount', 'root-mnt_nfs3.mount', 'root-mnt-nfs1.mount', 'root-mnt-nfs2.mount']
    assert ls.dirs_of(dir_path) == ['.', '..', 'local-fs.target.requires', 'nfs-server.service.d',
                               'remote-fs.target.requires', 'swap.target.requires']
    assert ls.listing_of(dir_path)['-.mount'] == {'type': '-', 'perms': 'rw-r--r--.', 'links': 1, 'owner': '0', 'group': '0', 'size': 217, 'date': 'Aug  5 07:35', 'name': '-.mount', 'raw_entry': '-rw-r--r--.  1 0 0 217 Aug  5 07:35 -.mount', 'dir': dir_path}


def test_ls_osroot_doc_examples():
    env = {
        'ls': LsRunSystemdGenerator(context_wrap(LS_RUN_SYSTEMD_GENERATOR,
                                    path=PATH_OF_LS_RUN_SYSTEMD_GENERATOR)),
    }
    failed, total = doctest.testmod(ls_run_systemd_generator, globs=env)
    assert failed == 0



"""
LsVarOptMssqlLog - command ``ls -la /var/opt/mssql/log``
========================================================

This parser reads the ``/var/opt/mssql/log`` directory listings and uses the
FileListing parser class to provide a common access to them.

"""

from insights import FileListing, parser, CommandParser
from insights.specs import Specs


@parser(Specs.ls_var_opt_mssql_log)
class LsVarOptMssqlLog(CommandParser, FileListing):
    """
    A parser for accessing "ls -la /var/opt/mssql/log".

    Examples:
        >>> '/var/opt/mssql/log' in ls_mssql_log
        True
        >>> ls_mssql_log.dir_contains('/var/opt/mssql/log', 'messages')
        False
    """
    pass



import pytest

from insights.core import JSONParser, ParseException
from insights.tests import context_wrap


class MyJsonParser(JSONParser):
    pass

json_test_strings = {
    '{"a": "1", "b": "2"}': {'a': '1', 'b': '2'},
    '[{"a": "1", "b": "2"},{"a": "3", "b": "4"}]':
        [{'a': '1', 'b': '2'}, {'a': '3', 'b': '4'}]
}


def test_json_parser_success():
    for jsonstr in json_test_strings:
        ctx = context_wrap(jsonstr)
        assert MyJsonParser(ctx).data == json_test_strings[jsonstr]


def test_json_parser_failure():
    ctx = context_wrap("boom")
    with pytest.raises(ParseException) as ex:
        MyJsonParser(ctx)

    assert "MyJsonParser" in ex.value.args[0]



from insights.parsers.package_provides_java import PackageProvidesJava
from insights.tests import context_wrap
from ...parsers import ParseException, SkipException
import pytest

PACKAGE_COMMAND_MATCH = """
/usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64
"""

PACKAGE_COMMAND_ERROR = """
"""

PACKAGE_COMMAND_NOT_MATCH = """
jdk-9/bin/java file /root/jdk-9/bin/java is not owned by any package
"""


def test_package_provides_java_match():
    package = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH))
    assert package.command == "/usr/lib/jvm/jre/bin/java"
    assert package.package == "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"


def test_package_provides_java_err():
    with pytest.raises(ParseException) as pe:
        PackageProvidesJava(context_wrap(PACKAGE_COMMAND_ERROR))
        assert "there is not java application running" in str(pe)


def test_package_provides_java_not_match():
    with pytest.raises(SkipException) as pe:
        PackageProvidesJava(context_wrap(PACKAGE_COMMAND_NOT_MATCH))
        assert "current running java command is not provided by package installed through yum or rpm" in str(pe)



"""
LsDVarOptMSSql - command ``ls -ld /var/opt/mssql``
==================================================
"""
from insights.specs import Specs
from .. import CommandParser, parser, FileListing


@parser(Specs.ls_var_opt_mssql)
class LsDVarOptMSSql(CommandParser, FileListing):
    """Parses output of ``ls -ld /var/opt/mssql`` command.

    The ``ls -ld /var/opt/mssql`` command provides information for the listing
    of the ``/var/opt/mssql`` directory. See ``FileListing`` class for addtional
    information.


    Sample ``ls -ld /var/opt/mssql`` output::

        drwxrwx---. 5 root root 58 Apr 16 07:20 /var/opt/mssql

    Examples:

        >>> content.listing_of('/var/opt/mssql').get('/var/opt/mssql').get('owner')
        'root'
        >>> content.listing_of('/var/opt/mssql').get('/var/opt/mssql').get('group')
        'root'
    """
    pass



import doctest
import pytest
from insights.tests import context_wrap
from insights.parsers import md5check
from insights.parsers import ParseException

NORMAL_MD5_SAMPLE = """
7d4855248419b8a3ce6616bbc0e58301  /etc/localtime
""".strip()

BAD_INPUT_SAMPLE = """
e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855  /dev/null
""".strip()

BAD_INPUT_MIX = """
d41d8cd98f00b204e9800998ecf8427e  /dev/null
/usr/bin/md5sum: /etc/pki/product/69.pem: No such file or directory
/usr/bin/md5sum: /etc/pki/product-default/69.pem: No such file or directory
""".strip()

BLANK_INPUT_SAMPLE = """
""".strip()


def test_normal_md5():
    md5info = md5check.NormalMD5(context_wrap(NORMAL_MD5_SAMPLE))
    assert md5info is not None
    assert md5info.filename == "/etc/localtime"
    assert md5info.md5sum == "7d4855248419b8a3ce6616bbc0e58301"


def test_normal_md5_blank_input():
    ctx = context_wrap(BLANK_INPUT_SAMPLE)
    with pytest.raises(ParseException) as sc:
        md5check.NormalMD5(ctx)
    assert "Incorrect length for input" in str(sc)


def test_normal_md5_bad_input():
    ctx = context_wrap(BAD_INPUT_SAMPLE)
    with pytest.raises(ParseException) as sc:
        md5check.NormalMD5(ctx)
    assert "Invalid MD5sum value" in str(sc)


def test_normal_md5_bad_mix():
    ctx = context_wrap(BAD_INPUT_MIX)
    with pytest.raises(ParseException) as sc:
        md5check.NormalMD5(ctx)
    assert "Incorrect length for input" in str(sc)


def test_md5_docs():
    env = {'md5info': md5check.NormalMD5(context_wrap(NORMAL_MD5_SAMPLE))}
    failed, total = doctest.testmod(md5check, globs=env)
    assert failed == 0



from insights.parsers.ifcfg import IfCFG
from insights.tests import context_wrap
from insights.util import keys_in


CONTEXT_PATH_DEVICE = "etc/sysconfig/network-scripts/ifcfg-eth0"

IFCFG_TEST_SPACE_V1 = """
DEVICE='"badName1"  '
BOOTPROTO=dhcp
IPV4_FAILURE_FATAL = no  #this is a comment
ONBOOT=yes
""".strip()

IFCFG_TEST_SPACE_V2 = """
DEVICE="\"badName2\"  "
BOOTPROTO=dhcp
IPV4_FAILURE_FATAL = no  #this is a comment
ONBOOT=yes
""".strip()

IFCFG_TEST_MASTER = """
DEVICE="eth0"
BOOTPROTO=dhcp
MASTER="bond0"
ONBOOT=yes
""".strip()

CONTEXT_PATH = "etc/sysconfig/network-scripts/ifcfg-enp0s25"

IFCFG_TEST = """
TYPE = "Ethernet"
BOOTPROTO=dhcp
IPV4_FAILURE_FATAL = no  #this is a comment
#ONBOOT=yes
NAME = enp0s25
""".strip()

IFCFG_TEST_2 = """
TYPE=Ethernet
BOOTPROTO=dhcp
DEFROUTE=yes
UUID=284549c8-0e07-41d3-a1e8-91ac9a9fca75
HWADDR=00:50:56:89:0B:B0
PEERDNS=yes
~
""".strip()

IFCFG_PATH_2 = "ssocommand/etc/sysconfig/network-scripts/ifcfg-=eno1"

IFCFG_TEST_3 = """
DEVICE=team1
DEVICETYPE=Team
ONBOOT=yes
NETMASK=255.255.252.0
IPADDR=192.168.0.1
TEAM_CONFIG='{"runner": {"name": "lacp", "active": "true", "tx_hash": ["eth", "ipv4"]}, "tx_balancer": {"name": "basic"}, "link_watch": {"name": "ethtool"}}'
""".strip()

IFCFG_PATH_3 = "etc/sysconfig/network-scripts/ifcfg-team1"

IFCFG_TEST_4 = """
DEVICE==eno2
ONBOOT=no
BOOTPROTO=none
USERCTL=no
DEVICETYPE=TeamPort
TEAM_MASTER=team1
TEAM_PORT_CONFIG='{"prio": 100}'
""".strip()

IFCFG_PATH_4 = "etc/sysconfig/network-scripts/ifcfg-=eno2"

IFCFG_TEST_5 = """
TEAM_PORT_CONFIG="{\\"prio\\": -10, \\"sticky\\": true, \\"link_watch\\": {\\"name\\": \\"ethtool\\"}}"
NAME=eth0
UUID=a4bd7fbc-3905-4ff1-a467-5b56c32572df
DEVICE=eth0
ONBOOT=yes
TEAM_MASTER=heartbeat
DEVICETYPE=TeamPort
ZONE=internal
""".strip()

IFCFG_PATH_5 = "etc/sysconfig/network-scripts/ifcfg-en0"

IFCFG_TEST_6 = """
DEVICE=bond0
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
BONDING_OPTS="mode=1 primary=eth1 arp_interval=1000 arp_ip_target=+10.11.96.1"


""".strip()

IFCFG_PATH_6 = "etc/sysconfig/network-scripts/ifcfg-en0"

IFCFG_TEST_NAMED_BOND_MODE = """
DEVICE=bond0
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
BONDING_OPTS="mode=balance-xor primary=eth1 arp_interval=1000 arp_ip_target=+10.11.96.1 downdelay =0"
""".strip()

IFCFG_TEST_RAW_MASTER_VALUE = """
DEVICE="eth2"
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
MASTER="bond0"
""".strip()

IFCFG_TEST_RAW_TEAM_MASTER_VALUE = """
TYPE=Ethernet
DEVICE="eth1"
BOOTPROTO=none
ONBOOT=yes
TEAM_MASTER="team0"
MTU=9000
""".strip()

IFCFG_TEST_RAW_BONDING_VALUE = """
DEVICE="bond0"
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
BONDING_OPTS="mode=balance-xor primary=eth1 arp_interval=1000 arp_ip_target=+10.11.96.1 downdelay =0"
""".strip()

IFCFG_CONFIG_STR_ERROR = """
DEVICE=bond0
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
BONDING_OPTS="mode=balance-xor primary = eth1 arp_interval= 1000 arp_ip_target=+10.11.96.1 downdelay =0"
""".strip()

IFCFG_PATH_NAMED_BOND_MODE = "etc/sysconfig/network-scripts/ifcfg-en0"

IFCFG_PATH_ETH1 = "etc/sysconfig/network-scripts/ifcfg-eth1"

IFCFG_PATH_ETH2 = "etc/sysconfig/network-scripts/ifcfg-eth2"

IFCFG_TEST_BADLY_NAMED_BOND_MODE = """
DEVICE=bond0
IPADDR=10.11.96.172
NETMASK=255.255.252.0
BOOTPROTO=none
ONBOOT=yes
USERCTL=no
IPV6INIT=no
BONDING_OPTS="mode=failover primary=eth1 arp_interval=1000 arp_ip_target=+10.11.96.1"
""".strip()

IFCFG_PATH_BADLY_NAMED_BOND_MODE = "etc/sysconfig/network-scripts/ifcfg-en0"

IFCFG_BLANK_LINE = """

DEVICE==eno2

ONBOOT=no
BOOTPROTO=none
USERCTL=no
DEVICETYPE=TeamPort
TEAM_MASTER=team1
TEAM_PORT_CONFIG='{"prio": 100}'
"""

IFCFG_PATH_BLANK_LINE = "etc/sysconfig/network-scripts/ifcfg-=eno2"


def test_ifcfg_space_v1():
    context = context_wrap(IFCFG_TEST_SPACE_V1)
    context.path = CONTEXT_PATH_DEVICE

    r = IfCFG(context)
    assert keys_in(["DEVICE", "iface", "ONBOOT", "BOOTPROTO",
                    "IPV4_FAILURE_FATAL", "raw_device_value"], r)
    assert r["DEVICE"] != '\'"badName1"  \''
    assert r["raw_device_value"] == '\'"badName1"  \''
    assert r["DEVICE"] == 'badName1'


def test_ifcfg_space_v2():
    context = context_wrap(IFCFG_TEST_SPACE_V2)
    context.path = CONTEXT_PATH_DEVICE

    r = IfCFG(context)
    assert keys_in(["DEVICE", "iface", "ONBOOT", "BOOTPROTO",
                    "IPV4_FAILURE_FATAL", "raw_device_value"], r)
    assert r["DEVICE"] != '\"\"badName2\"  \"'
    assert r["raw_device_value"] == '\"\"badName2\"  \"'
    assert r["DEVICE"] == 'badName2'


def test_ifcfg_master():
    context = context_wrap(IFCFG_TEST_MASTER)
    context.path = CONTEXT_PATH_DEVICE

    r = IfCFG(context)
    assert keys_in(["DEVICE", "iface", "ONBOOT", "BOOTPROTO",
                    "MASTER", "raw_device_value"], r)
    assert r["MASTER"] == 'bond0'


def test_ifcfg():
    context = context_wrap(IFCFG_TEST)
    context.path = CONTEXT_PATH

    r = IfCFG(context)
    assert keys_in(["iface", "TYPE", "BOOTPROTO",
                    "IPV4_FAILURE_FATAL", "NAME"], r)
    assert not keys_in(["ONBOOT"], r)
    assert r["TYPE"] == "Ethernet"
    assert r["BOOTPROTO"] == "dhcp"
    assert r["IPV4_FAILURE_FATAL"] == "no"
    assert r["NAME"] == "enp0s25"
    assert r["iface"] == "enp0s25"
    assert r.ifname == r['iface']
    assert r.bonding_mode is None


def test_ifcfg_2():
    context = context_wrap(IFCFG_TEST_2)
    context.path = IFCFG_PATH_2

    r = IfCFG(context)
    assert len(r.data) == 7
    assert r["iface"] == "=eno1"
    assert r["TYPE"] == "Ethernet"
    assert r["BOOTPROTO"] == "dhcp"
    assert r["DEFROUTE"] == "yes"
    assert r["UUID"] == "284549c8-0e07-41d3-a1e8-91ac9a9fca75"
    assert r["HWADDR"] == "00:50:56:89:0B:B0"
    assert r["PEERDNS"] == "yes"


def test_ifcfg_3():
    context = context_wrap(IFCFG_TEST_3)
    context.path = IFCFG_PATH_3

    r = IfCFG(context)

    assert len(r.data) == 8
    assert r["DEVICE"] == "team1"
    assert r["DEVICETYPE"] == "Team"
    assert r["ONBOOT"] == "yes"
    assert r["NETMASK"] == "255.255.252.0"
    assert r["IPADDR"] == "192.168.0.1"
    assert r["TEAM_CONFIG"]["runner"]["name"] == "lacp"
    assert r["TEAM_CONFIG"]["runner"]["active"] == "true"
    assert r["TEAM_CONFIG"]["tx_balancer"]["name"] == "basic"
    assert r["TEAM_CONFIG"]["link_watch"]["name"] == "ethtool"


def test_ifcfg_4():
    context = context_wrap(IFCFG_TEST_4)
    context.path = IFCFG_PATH_4

    r = IfCFG(context)

    assert r["TEAM_PORT_CONFIG"]["prio"] == 100
    assert r["iface"] == "=eno2"
    assert r["DEVICE"] == "=eno2"


def test_ifcfg_5():
    context = context_wrap(IFCFG_TEST_5)
    context.path = IFCFG_PATH_5

    r = IfCFG(context)

    assert r["TEAM_PORT_CONFIG"]["prio"] == -10


def test_ifcfg_6():
    context = context_wrap(IFCFG_TEST_6)
    context.path = IFCFG_PATH_6

    r = IfCFG(context)

    assert r["BONDING_OPTS"]["mode"] == "1"
    assert r["BONDING_OPTS"]["arp_ip_target"] == "+10.11.96.1"
    assert r.bonding_mode == 1


def test_ifcfg_named_bond_mode():
    context = context_wrap(IFCFG_TEST_NAMED_BOND_MODE)
    context.path = IFCFG_PATH_NAMED_BOND_MODE

    r = IfCFG(context)

    assert r.bonding_mode == 2


def test_ifcfg_badly_named_bond_mode():
    context = context_wrap(IFCFG_TEST_BADLY_NAMED_BOND_MODE)
    context.path = IFCFG_PATH_BADLY_NAMED_BOND_MODE

    r = IfCFG(context)

    assert r.bonding_mode is None


def test_ifcfg_bonding_opts():
    context = context_wrap(IFCFG_TEST_NAMED_BOND_MODE)
    context.path = IFCFG_PATH_NAMED_BOND_MODE

    r = IfCFG(context)

    assert r["BONDING_OPTS"]["mode"] == "balance-xor"
    assert r["BONDING_OPTS"]["arp_ip_target"] == "+10.11.96.1"
    assert r["BONDING_OPTS"]["downdelay"] == "0"
    assert r.has_empty_line is False
    assert r.bonding_mode == 2


def test_ifcfg_blankline():
    context = context_wrap(IFCFG_BLANK_LINE)
    context.path = IFCFG_PATH_BLANK_LINE
    r = IfCFG(context)
    assert r.has_empty_line is True


def test_ifcfg_raw_bonding_master_value():
    context = context_wrap(IFCFG_TEST_RAW_MASTER_VALUE)
    context.path = IFCFG_PATH_ETH2

    r = IfCFG(context)
    assert r["raw_device_value"] == '"eth2"'
    assert r["raw_master_value"] == '"bond0"'


def test_ifcfg_raw_bonding_value():
    context = context_wrap(IFCFG_TEST_RAW_BONDING_VALUE)
    context.path = IFCFG_PATH_NAMED_BOND_MODE

    r = IfCFG(context)

    assert r["raw_device_value"] == '"bond0"'
    assert r["raw_bonding_value"] == '"mode=balance-xor primary=eth1 arp_interval=1000 arp_ip_target=+10.11.96.1 downdelay =0"'


def test_ifcfg_raw_team_master_value():
    context = context_wrap(IFCFG_TEST_RAW_TEAM_MASTER_VALUE)
    context.path = IFCFG_PATH_ETH1

    r = IfCFG(context)

    assert r["raw_device_value"] == '"eth1"'
    assert r["raw_team_value"] == '"team0"'



import doctest
from doctest import (DebugRunner, DocTestFinder, DocTestRunner,
                     OutputChecker)
import re
import sys


class Py23DocChecker(OutputChecker):
    def check_output(self, want, got, optionflags):
        if sys.version_info[0] > 2:
            want = re.sub("u'(.*?)'", "'\\1'", want)
            want = re.sub("u'(.*?)'", "'\\1'", want)
        return OutputChecker.check_output(self, want, got, optionflags)


def ic_testmod(m, name=None, globs=None, verbose=None,
               report=True, optionflags=0, extraglobs=None,
               raise_on_error=False, exclude_empty=False):
    """See original code in doctest.testmod."""
    if name is None:
        name = m.__name__
    finder = DocTestFinder(exclude_empty=exclude_empty)
    if raise_on_error:
        runner = DebugRunner(checker=Py23DocChecker(),
                             verbose=verbose,
                             optionflags=optionflags)
    else:
        runner = DocTestRunner(checker=Py23DocChecker(),
                               verbose=verbose,
                               optionflags=optionflags)
    for test in finder.find(m, name, globs=globs, extraglobs=extraglobs):
        runner.run(test)

    if report:
        runner.summarize()

    return doctest.TestResults(runner.failures, runner.tries)



"""
VirtWhoConf - File ``/etc/virt-who.conf`` and ``/etc/virt-who.d/*.conf``
========================================================================

The ``VirtWhoConf`` class parses the virt-who configuration files in `ini-like`
format.

    .. note::

        The configuration files under ``/etc/virt-who.d/`` might contain
        sensitive information, like ``password``. It must be filtered.
"""

from .. import parser, LegacyItemAccess, IniConfigFile, add_filter
from insights.specs import Specs

filter_list = [
        '[',
        'interval',
        'oneshot',
        'type',
        'server',
        'debug',
        'log_',
        'configs',
        'owner',
        'env',
]
add_filter(Specs.virt_who_conf, filter_list)


@parser(Specs.virt_who_conf)
class VirtWhoConf(LegacyItemAccess, IniConfigFile):
    """
    Parse the ``virt-who`` configuration files ``/etc/virt-who.conf`` and
    ``/etc/virt-who.d/*.conf``.

    Sample configuration file::

        ## This is a template for virt-who global configuration files. Please see
        ## virt-who-config(5) manual page for detailed information.
        ##
        ## virt-who checks /etc/virt-who.conf for sections 'global' and 'defaults'.
        ## The sections and their values are explained below.
        ## NOTE: These sections retain their special meaning and function only when present in /etc/virt-who.conf
        ##
        ## You can uncomment and fill following template or create new file with
        ## similar content.

        #Terse version of the general config template:
        [global]

        interval=3600
        #reporter_id=
        debug=False
        oneshot=False
        #log_per_config=False
        #log_dir=
        #log_file=
        #configs=

        [defaults]
        owner=Satellite
        env=Satellite

    Examples:

        >>> vwho_conf = shared[VirtWhoConf]
        >>> 'global' in vwho_conf
        True
        >>> vwho_conf.has_option('global', 'debug')
        True
        >>> vwho_conf.get('global', 'oneshot')
        "False"
        >>> vwho_conf.getboolean('global', 'oneshot')
        False
        >>> vwho_conf.get('global', 'interval')
        "3600"
        >>> vwho_conf.getint('global', 'interval')
        3600
        >>> vwho_conf.items('defaults')
        {'owner': 'Satellite', 'env': 'Satellite'}
    """
    pass



"""
NetworkManagerConfig - file ``/etc/NetworkManager/NetworkManager.conf``
=======================================================================

The ``/etc/NetworkManager/NetworkManager.conf`` file is in a standard '.ini' format,
and this parser uses the IniConfigFile base class to read this.

Given a file containing the following test data::

    [main]
    #plugins=ifcfg-rh,ibft
    dhcp=dhclient

Example:
    >>> type(networkmanager_config_obj)
    <class 'insights.parsers.networkmanager_config.NetworkManagerConfig'>
    >>> networkmanager_config_obj.get('main', 'dhcp') == 'dhclient'
    True
"""

from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.networkmanager_conf)
class NetworkManagerConfig(IniConfigFile):
    """
    A dict of the content of the ``NetworkManager.conf`` configuration file.

    Example selection of dictionary contents::

        {
            'main': {
                'dhcp':'dhclient',
             }
        }
    """
    pass



"""
UdevRules - file ``/usr/lib/udev/rules.d/``
===========================================

The parsers included in this module are:

UdevRulesFCWWPN - file ``/usr/lib/udev/rules.d/59-fc-wwpn-id.rules``
--------------------------------------------------------------------

UdevRules40Redhat - files ``/etc/udev/rules.d/40-redhat.rules``, ``/run/udev/rules.d/40-redhat.rules``, ``/usr/lib/udev/rules.d/40-redhat.rules``, ``/usr/local/lib/udev/rules.d/40-redhat.rules``
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
"""
from insights import parser
from insights.core import LogFileOutput
from insights.specs import Specs


@parser(Specs.udev_fc_wwpn_id_rules)
class UdevRulesFCWWPN(LogFileOutput):
    """
    Read the content of ``/usr/lib/udev/rules.d/59-fc-wwpn-id.rules`` file.

    .. note::

        The syntax of the `.rules` file is complex, and no rules require to
        get the serialized parsed result currently.  An only existing rule's
        supposed to check the syntax of some specific line, so here the
        :class:`insights.core.LogFileOutput` is the base class.

    Examples:
        >>> type(udev_rules)
        <class 'insights.parsers.udev_rules.UdevRulesFCWWPN'>
        >>> 'ENV{FC_TARGET_WWPN}!="$*"; GOTO="fc_wwpn_end"' in udev_rules.lines
        True
    """
    pass


@parser(Specs.etc_udev_40_redhat_rules)
class UdevRules40Redhat(LogFileOutput):
    """
    Read the content of ``40-redhat.rules`` file.

    .. note::

        The syntax of the `.rules` file is complex, and no rules require to
        get the serialized parsed result currently.  An only existing rule's
        supposed to check the syntax of some specific line, so here the
        :class:`insights.core.LogFileOutput` is the base class.

    Sample input::

        # do not edit this file, it will be overwritten on update
        # CPU hotadd request
        SUBSYSTEM=="cpu", ACTION=="add", TEST=="online", ATTR{online}=="0", ATTR{online}="1"

        # Memory hotadd request
        SUBSYSTEM!="memory", ACTION!="add", GOTO="memory_hotplug_end"
        PROGRAM="/bin/uname -p", RESULT=="s390*", GOTO="memory_hotplug_end"

        LABEL="memory_hotplug_end"

    Examples:
        >>> 'LABEL="memory_hotplug_end"' in udev_40_redhat_rules.lines
        True
    """
    pass



"""
corosync_conf parses corosync configuration files into nested dictionaries.
"""
import string
from insights.parsr import (EOF, Forward, InSet, LeftCurly, Lift,
        LineEnd, Literal, RightCurly, Many, Number, OneLineComment, PosMarker,
        skip_none, String, QuotedString, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section


def loads(data):
    return Entry(children=Top(data))


def load(f):
    return loads(f.read())


def to_entry(name, rest):
    if isinstance(rest, list):
        return Section(name=name.value, children=rest, lineno=name.lineno)
    return Directive(name=name.value, attrs=[rest], lineno=name.lineno)


Sep = InSet(":=")
Stmt = Forward()
Num = Number & (WSChar | LineEnd)
NULL = Literal("none", value=None)
Comment = (WS >> OneLineComment("#").map(lambda x: None))
BeginBlock = (WS >> LeftCurly << WS)
EndBlock = (WS >> RightCurly << WS)
Bare = String(set(string.printable) - (set(string.whitespace) | set("#{}'\"")))
Name = WS >> PosMarker(String(string.ascii_letters + "_" + string.digits)) << WS
Value = WS >> (Num | NULL | QuotedString | Bare) << WS
Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
Stanza = (Lift(to_entry) * Name * (Block | (Sep >> Value))) | Comment
Stmt <= WS >> Stanza << WS
Doc = Many(Stmt).map(skip_none)
Top = Doc << EOF



from insights.parsers import mdadm
from insights.parsers.mdadm import MDAdmMetadata
from insights.tests import context_wrap

import doctest

MDADM_CONTENT = """
/dev/loop0:
Magic : a92b4efc
Version : 1.0
Feature Map : 0x0
Array UUID : 98e098ef:c8662ce2:2ed2aa5f:7f0416a9
Name : 0
Creation Time : Mon Jun 29 02:16:52 2020
Raid Level : raid1
Raid Devices : 2

Avail Dev Size : 16383968 sectors (7.81 GiB 8.39 GB)
Array Size : 1048576 KiB (1024.00 MiB 1073.74 MB)
Used Dev Size : 2097152 sectors (1024.00 MiB 1073.74 MB)
Super Offset : 16383984 sectors
Unused Space : before=0 sectors, after=14286824 sectors
State : clean
Device UUID : 5e249ed9:a9ee800a:c09c963f:363a18d2

Update Time : Mon Jun 29 02:19:56 2020
Bad Block Log : 512 entries available at offset -8 sectors
Checksum : 395066e8 - correct
Events : 60

Device Role : Active device 0
Array State : AA ('A' == active, '.' == missing, 'R' == replacing)
"""  # noqa


def test_doc_examples():
    env = {
        'mdadm': MDAdmMetadata(context_wrap(
            MDADM_CONTENT, path='insights_commands/mdadm_-E_.dev.loop0'
        )),
    }
    failed, total = doctest.testmod(mdadm, globs=env)
    assert failed == 0


def test_mdadm():
    md = MDAdmMetadata(context_wrap(
        MDADM_CONTENT, path='insights_commands/mdadm_-E_.dev.loop0'
    ))

    # Device assertions
    assert md.device == '/dev/loop0'

    # Information assertions
    assert md['Update Time'] == 'Mon Jun 29 02:19:56 2020'
    assert md['Array Size'] == '1048576 KiB (1024.00 MiB 1073.74 MB)'



from .. import parser
from ..parsers.pam import PamDConf
from insights.specs import Specs


@parser(Specs.password_auth)
class PasswordAuthPam(PamDConf):
    """Parsing for `/etc/pam.d/password-auth`. """
    pass



"""
EtcdConf - file ``/etc/etcd/etcd.conf``
=======================================

The EtcdConf class parses the file ``/etc/etcd/etcd.conf``. The
etcd.conf is in the standard 'ini' format and is read by the base
parser class `IniConfigFile`.

Typical contents of the file look like::

    [member]
    ETCD_NAME=f05-h19-000-1029p.rdu2.scalelab.redhat.com
    ETCD_LISTEN_PEER_URLS=https://10.1.40.235:2380
    ETCD_DATA_DIR=/var/lib/etcd/
    #ETCD_WAL_DIR=
    #ETCD_SNAPSHOT_COUNT=10000
    ETCD_HEARTBEAT_INTERVAL=500
    ETCD_ELECTION_TIMEOUT=2500
    ETCD_LISTEN_CLIENT_URLS=https://10.1.40.235:2379
    #ETCD_MAX_SNAPSHOTS=5
    #ETCD_MAX_WALS=5
    #ETCD_CORS=


    #[cluster]
    ETCD_INITIAL_ADVERTISE_PEER_URLS=https://10.1.40.235:2380
    ETCD_INITIAL_CLUSTER=f05-h19-000-1029p.rdu2.scalelab.redhat.com=https://10.1.40.235:2380
    ETCD_INITIAL_CLUSTER_STATE=new
    ETCD_INITIAL_CLUSTER_TOKEN=etcd-cluster-1
    #ETCD_DISCOVERY=
    #ETCD_DISCOVERY_SRV=
    #ETCD_DISCOVERY_FALLBACK=proxy
    #ETCD_DISCOVERY_PROXY=
    ETCD_ADVERTISE_CLIENT_URLS=https://10.1.40.235:2379
    #ETCD_STRICT_RECONFIG_CHECK=false
    #ETCD_AUTO_COMPACTION_RETENTION=0
    #ETCD_ENABLE_V2=true
    ETCD_QUOTA_BACKEND_BYTES=8589934592

    #[proxy]
    #ETCD_PROXY=off
    #ETCD_PROXY_FAILURE_WAIT=5000
    #ETCD_PROXY_REFRESH_INTERVAL=30000
    #ETCD_PROXY_DIAL_TIMEOUT=1000
    #ETCD_PROXY_WRITE_TIMEOUT=5000
    #ETCD_PROXY_READ_TIMEOUT=0

    #[security]
    ETCD_TRUSTED_CA_FILE=/etc/etcd/ca.crt
    ETCD_CLIENT_CERT_AUTH=true
    ETCD_CERT_FILE=/etc/etcd/server.crt
    ETCD_KEY_FILE=/etc/etcd/server.key
    #ETCD_AUTO_TLS=false
    ETCD_PEER_TRUSTED_CA_FILE=/etc/etcd/ca.crt
    ETCD_PEER_CLIENT_CERT_AUTH=true
    ETCD_PEER_CERT_FILE=/etc/etcd/peer.crt
    ETCD_PEER_KEY_FILE=/etc/etcd/peer.key
    #ETCD_PEER_AUTO_TLS=false

    #[logging]
    ETCD_DEBUG=False

    #[profiling]
    #ETCD_ENABLE_PPROF=false
    #ETCD_METRICS=basic
    #
    [auth]
    ETCD_AUTH_TOKEN=simple

Examples:

    >>> conf.get('auth', 'ETCD_AUTH_TOKEN') == 'simple'
    True

    >>> conf.has_option('member', 'ETCD_NAME')
    True
"""


from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

add_filter(Specs.etcd_conf, ["["])


@parser(Specs.etcd_conf)
class EtcdConf(IniConfigFile):
    """Class for etcd.conf file content."""
    pass



"""
NscdConf - file ``/etc/nscd.conf``
==================================

This module parses the contents of the file ``/etc/nscd.conf``.

Each line of the ``nscd.conf`` file specifies either an attribute and a value,
or an attribute, service, and a value. Fields are separated either by SPACE
or TAB characters.  A '#' (number sign) indicates the beginning of a comment;
following characters, up to the end of the line, are not interpreted by nscd.

The function ``service_attributes`` provides information on the lines that
contain a service column such as `passwd` and `group` in the examples below.
The function ``attribute`` can be used to obtain values for attributes not
associated with a service such as `server-user` and `debug-level` in the
examples.

Sample content of the ``nscd.conf`` file looks like::

    #       logfile                 /var/log/nscd.log
    #       threads                 4
    #       max-threads             32
            server-user             nscd
    #       stat-user               somebody
            debug-level             0
    #       reload-count            5
            paranoia                no
    #       restart-interval        3600

            enable-cache            passwd          no
            positive-time-to-live   passwd          600
            negative-time-to-live   passwd          20
            suggested-size          passwd          211
            check-files             passwd          yes
            persistent              passwd          yes
            shared                  passwd          yes
            max-db-size             passwd          33554432
            auto-propagate          passwd          yes

            enable-cache            group           no
            positive-time-to-live   group           3600
            negative-time-to-live   group           60
            suggested-size          group           211
            check-files             group           yes
            persistent              group           yes
            shared                  group           yes
            max-db-size             group           33554432
            auto-propagate          group           yes

Examples:
    >>> conf = shared[NscdConf]
    >>> len([line for line in conf])
    21
    >>> [line for line in conf][0]
    NscdConfLine(attribute='server-user', service=None, value='nscd')
    >>> conf.attribute("server-user")
    'nscd'
    >>> conf.filter("server-user")
    [NscdConfLine(attribute='server-user', service=None, value='nscd')]
    >>> conf.filter("server-user")[0].attribute
    'server-user'
    >>> conf.filter("server-user")[0].value
    'nscd'
    >>> conf.service_attributes("passwd")
    [NscdConfLine(attribute='enable-cache', service='passwd', value='no'),
     NscdConfLine(attribute='positive-time-to-live', service='passwd', value='600'),
     NscdConfLine(attribute='negative-time-to-live', service='passwd', value='20'),
     NscdConfLine(attribute='suggested-size', service='passwd', value='211'),
     NscdConfLine(attribute='check-files', service='passwd', value='yes'),
     NscdConfLine(attribute='persistent', service='passwd', value='yes'),
     NscdConfLine(attribute='shared', service='passwd', value='yes'),
     NscdConfLine(attribute='max-db-size', service='passwd', value='33554432'),
     NscdConfLine(attribute='auto-propagate', service='passwd', value='yes')]
    >>> conf.filter("enable-cache")
    [NscdConfLine(attribute='enable-cache', service='passwd', value='no'),
     NscdConfLine(attribute='enable-cache', service='group', value='no')]
    >>> conf.filter("enable-cache", service="passwd")
    [NscdConfLine(attribute='enable-cache', service='passwd', value='no')]
    >>> conf.filter("enable-cache", service="passwd")[0].attribute
    'enable-cache'
    >>> conf.filter("enable-cache", service="passwd")[0].service
    'passwd'
    >>> conf.filter("enable-cache", service="passwd")[0].value
    'no'
"""
from collections import defaultdict, namedtuple
from .. import Parser, parser, get_active_lines
from insights.specs import Specs

NscdConfLine = namedtuple('NscdConfLine', ['attribute', 'service', 'value'])
"""namedtuple: Represents one line of information from the conf file."""


@parser(Specs.nscd_conf)
class NscdConf(Parser):
    """Class for parsing contents of the ``/etc/nscd.conf`` file."""
    def parse_content(self, content):
        self.data = []
        for line in get_active_lines(content):
            parts = line.split()
            if len(parts) == 2:
                self.data.append(NscdConfLine(parts[0], None, parts[1]))
            else:
                self.data.append(NscdConfLine(parts[0], parts[1], parts[2]))

        self._attributes = {}
        services = defaultdict(list)
        for datum in self.data:
            if datum.service:
                services[datum.service].append(datum)
            else:
                self._attributes[datum.attribute] = datum.value
        self._services = dict(services)

    def service_attributes(self, service_name):
        """list: Returns a list of conf lines matching ``service_name``."""
        return self._services.get(service_name, [])

    def attribute(self, attribute):
        """str: Returns the value of attribute with name ``attribute``.
        Lines that include a ``service`` are not returned by this function.
        """
        return self._attributes.get(attribute)

    def filter(self, attribute, service=None):
        """list: Returns list of conf lines containing ``attribute`` and
        optional ``service`` if present.
        """
        return [conf for conf in self.data
                if attribute in conf.attribute and
                (not service or (service and service in conf.service))]

    def __iter__(self):
        for line in self.data:
            yield line



from insights.parsr.examples.logrotate_conf import loads

EXAMPLE = """
# sample logrotate configuration file
compress

/var/log/messages {
    rotate 5
    weekly
    postrotate
        /usr/bin/killall -HUP syslogd
    endscript
}

"/var/log/httpd/access.log" /var/log/httpd/error.log {
    rotate 5
    mail www@my.org
    size 100k
    sharedscripts
    postrotate
        /usr/bin/killall -HUP httpd
    endscript
}

/var/log/news/* {
    monthly
    rotate 2
    olddir /var/log/news/old
    missingok
    postrotate
        kill -HUP 'cat /var/run/inn.pid'
    endscript
    nocompress
}
""".strip()


SIMPLE = """
# sample logrotate configuration file
compress

 /var/log/messages {
    rotate 5
    weekly
    postrotate
        /usr/bin/killall -HUP syslogd
    endscript
}
"""


def test_logrotate_simple():
    res = loads(SIMPLE)
    assert "compress" in res
    assert "/var/log/messages" in res


def test_logrotate_example():
    res = loads(EXAMPLE)
    assert res["compress"].value is None
    assert res["/var/log/messages"]["rotate"].value == 5
    assert res["/var/log/messages"]["rotate"][0].lineno == 5
    assert res["/var/log/messages"]["weekly"].value is None
    assert res["/var/log/messages"]["postrotate"].value == "/usr/bin/killall -HUP syslogd"
    assert res["/var/log/news/*"][0].lineno == 22


def xtest_logrotate_multikey():
    res = loads(EXAMPLE)
    assert res["compress"].value is None
    assert res["/var/log/httpd/access.log"]["rotate"].value == 5
    assert res["/var/log/httpd/access.log"]["size"].value == "100k"
    assert res["/var/log/httpd/access.log"]["sharedscripts"].value is None

    assert res["/var/log/httpd/error.log"]["rotate"].value == 5
    assert res["/var/log/httpd/error.log"]["size"].value == "100k"
    assert res["/var/log/httpd/error.log"]["sharedscripts"].value is None
    assert res["/var/log/news/*"]["postrotate"].value == "kill -HUP 'cat /var/run/inn.pid'"



from insights.parsr.query import Entry


complex_tree = Entry(name="root",
                    attrs=[1, 2, 3, 4],
                    children=[
                        Entry(name="child", attrs=[1, 1, 2]),
                        Entry(name="child", attrs=[1, 1, 2, 3, 5]),
                        Entry(name="child", attrs=[1, 1, 3, 5, 9]),
                        Entry(name="dog", attrs=["woof"], children=[
                            Entry(name="puppy", attrs=["smol"]),
                            Entry(name="puppy", attrs=["fluffy"]),
                            Entry(name="kitten", attrs=["wut"]),
                        ])
                    ])


def test_select_leaves():
    res = complex_tree.select("puppy", deep=True)
    assert len(res) == 2
    assert res[0].name == "puppy"
    assert res[1].name == "puppy"


def test_select_roots():
    res = complex_tree.select("puppy", deep=True, roots=True)
    assert len(res) == 1
    assert res[0].name == "root"


def test_select_chain():
    res = complex_tree.select("dog").select("puppy")
    assert len(res) == 2



"""
OVSvsctlListBridge - command ``/usr/bin/ovs-vsctl list bridge``
===============================================================

This module provides class ``OVSvsctlListBridge`` for parsing the
output of command ``ovs-vsctl list bridge``.
"""

from insights import CommandParser, get_active_lines, parser
from insights.parsers import SkipException, optlist_to_dict
from insights.specs import Specs


@parser(Specs.ovs_vsctl_list_bridge)
class OVSvsctlListBridge(CommandParser):
    """
    Class to parse output of command ``ovs-vsctl list bridge``.
    Generally, the data is in key:value format with values having
    data types as string, numbers, list or dictionary.
    The class provides attribute ``data`` as list with lines parsed
    line by line based on keys for each bridge.

    Sample command output::

        name                : br-int
        other_config        : {disable-in-band="true", mac-table-size="2048"}
        name                : br-tun
        other_config        : {}

    Examples:
        >>> bridge_lists[0]["name"]
        'br-int'
        >>> bridge_lists[0]["other_config"]["mac-table-size"]
        '2048'
        >>> bridge_lists[0]["other_config"]["disable-in-band"]
        'true'
        >>> bridge_lists[1].get("name")
        'br-tun'
        >>> len(bridge_lists[1]["other_config"]) == 0
        True

    Attributes:
        data (list): A list containing dictionary elements where each
                     element contains the details of a bridge.

    Raises:
        SkipException: When file is empty.
    """

    bridge_keys = ("_uuid", "auto_attach", "controller", "datapath_id",
            "datapath_type", "datapath_version", "external_ids", "fail_mode",
            "flood_vlans", "flow_tables", "ipfix", "mcast_snooping_enable:",
            "mirrors", "name", "netflow", "other_config", "ports", "protocols",
            "rstp_enable", "rstp_status", "sflow", "status", "stp_enable")

    def parse_content(self, content):
        """
           Details of all the bridges are extracted and stored in a list as dictionary
           elements. Each dictionary element contains the information of a specific
           bridge.
        """
        # No content found or file is empty
        if not content:
            raise SkipException("Empty file")

        self.data = []
        bridge_details = {}
        for line in get_active_lines(content):
            key, value = [i.strip() for i in line.split(":", 1)]
            parsed_value = value.strip('"')
            if value.startswith("{") and value.endswith("}"):
                parsed_value = {}
                value = value.strip("{}")
                if value:
                    parsed_value = optlist_to_dict(value, opt_sep=", ", strip_quotes=True)
            elif value.startswith("[") and value.endswith("]"):
                parsed_value = []
                value = value.strip("[]")
                if value:
                    parsed_value = [i.strip(' \t\"\'') for i in value.split(",")]

            if key not in bridge_details:
                bridge_details[key] = parsed_value
            else:
                # A new bridge comes
                self.data.append(bridge_details)
                bridge_details = {key: parsed_value}
        # Add the last bridge
        self.data.append(bridge_details)

    def __getitem__(self, line):
        return self.data[line]



"""
SystemdAnalyzeBlame - command ``systemd-analyze blame``
=======================================================

This module parses the output of command ``systemd-analyze blame``.
"""
from insights.specs import Specs
from insights import CommandParser, parser
from insights.parsers import SkipException


@parser(Specs.systemd_analyze_blame)
class SystemdAnalyzeBlame(CommandParser, dict):
    """Parse the output of ``systemd-analyze blame`` as ``dict``. The time to
    initialize is converted into seconds.

    Typical output::

        33.080s cloud-init-local.service
        32.423s unbound-anchor.service
         2.773s kdump.service
         1.699s dnf-makecache.service
         1.304s cloud-init.service
         1.073s initrd-switch-root.service
          939ms cloud-config.service
          872ms tuned.service
          770ms cloud-final.service

    Examples:

        >>> 'cloud-init-local.service' in output
        True
        >>> output.get('cloud-init.service', 0)
        1.304

    Returns:
        (dict): With unit-name & time as key-value pair.
                Ex::

                    {'cloud-config.service': 0.939,
                     'cloud-final.service': 0.77,
                     'cloud-init-local.service': 33.08,
                     'cloud-init.service': 1.304,
                     'dnf-makecache.service': 1.699,
                     'initrd-switch-root.service': 1.073,
                     'kdump.service': 2.773,
                     'tuned.service': 0.872,
                     'unbound-anchor.service': 32.423}

    Raises:
        SkipException: If content is not provided.
    """
    def parse_content(self, content):
        if not content:
            raise SkipException

        for c in content:
            time, service = c.split()
            if time.endswith('ms'):
                _time = round(float(time.strip('ms')) / 1000, 5)
            else:
                _time = round(float(time.strip('ms')), 5)

            self[service] = _time



"""
PuppetserverConfig - file ``/etc/sysconfig/puppetserver``
=========================================================
"""
from insights.specs import Specs
from insights.util import deprecated
from . import split_kv_pairs
from .. import LegacyItemAccess, Parser, get_active_lines, parser


@parser(Specs.puppetserver_config)
class PuppetserverConfig(Parser, LegacyItemAccess):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.PuppetserverSysconfig` instead.

    Parse the puppetserver configuration file.

    Produces a simple dictionary of keys and values from the configuration
    file contents , stored in the ``data`` attribute.  The object also
    functions as a dictionary itself thanks to the
    :py:class:`insights.core.LegacyItemAccess` mixin class.

    Sample configuration file::

        ###########################################
        # Init settings for puppetserver
        ###########################################

        # Location of your Java binary (version 7 or higher)
        JAVA_BIN="/usr/bin/java"

        # Modify this if you'd like to change the memory allocation, enable JMX, etc
        JAVA_ARGS="-Xms2g -Xmx2g -XX:MaxPermSize=256m"

        # These normally shouldn't need to be edited if using OS packages
        USER="puppet"
        GROUP="puppet"
        INSTALL_DIR="/opt/puppetlabs/server/apps/puppetserver"
        CONFIG="/etc/puppetlabs/puppetserver/conf.d"

        BOOTSTRAP_CONFIG="/etc/puppetlabs/puppetserver/services.d/,/opt/puppetlabs/server/apps/puppetserver/config/services.d/"

        SERVICE_STOP_RETRIES=60

        START_TIMEOUT=300

        RELOAD_TIMEOUT=120

    Examples:
        >>> puppetserver_config['START_TIMEOUT']
        '300'
        >>> 'AUTO' in puppetserver_config
        False
    """
    def __init__(self, *args, **kwargs):
        deprecated(PuppetserverConfig, "Import PuppetserverSysconfig from insights.parsers.sysconfig instead")
        super(PuppetserverConfig, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.data = split_kv_pairs(get_active_lines(content))



from insights.core.context import (ExecutionContextMeta, HostArchiveContext,
                                   SerializedArchiveContext, SosArchiveContext)


def test_host_archive_context():
    files = ["/foo/junk", "/insights_commands"]
    actual = HostArchiveContext.handles(files)
    assert actual == ("/", HostArchiveContext), actual

    files = ["/foo/junk", "/insights_commands/things"]
    actual = HostArchiveContext.handles(files)
    assert actual == ("/", HostArchiveContext), actual

    files = ["/foo/junk", "/foo/junk/insights_commands/foobar.txt"]
    actual = HostArchiveContext.handles(files)
    assert actual == ("/foo/junk", HostArchiveContext), actual


def test_host_archive_context_unsupported():
    files = ["/foo/junk", "/not_insights_commands"]
    actual = HostArchiveContext.handles(files)
    assert actual == (None, None), actual

    files = ["/foo/junk", "/insights_commands_not"]
    actual = HostArchiveContext.handles(files)
    assert actual == (None, None), actual


def test_sos_archive_context_supported():
    files = ["/foo/junk", "/sos_commands"]
    actual = SosArchiveContext.handles(files)
    assert actual == ("/", SosArchiveContext), actual

    files = ["/foo/junk", "/sos_commands/things"]
    actual = SosArchiveContext.handles(files)
    assert actual == ("/", SosArchiveContext), actual

    files = ["/foo/junk", "/foo/junk/sos_commands/foobar.txt"]
    actual = SosArchiveContext.handles(files)
    assert actual == ("/foo/junk", SosArchiveContext), actual


def test_sos_archive_context_unsupported():
    files = ["/foo/junk", "/sos_commands_not"]
    actual = SosArchiveContext.handles(files)
    assert actual == (None, None), actual

    files = ["/foo/junk", "/not_sos_commands"]
    actual = SosArchiveContext.handles(files)
    assert actual == (None, None), actual


def test_serialize_archive_context_supported():
    files = ["/foo/junk", "/insights_archive.txt"]
    actual = SerializedArchiveContext.handles(files)
    assert actual == ("/", SerializedArchiveContext), actual


def test_serialized_archive_context_unsupported():
    files = ["/foo/junk", "/sos_commands_not"]
    actual = SerializedArchiveContext.handles(files)
    assert actual == (None, None), actual

    files = ["/foo/junk", "/insights_archive"]
    actual = SerializedArchiveContext.handles(files)
    assert actual == (None, None), actual


def test_unrecognized():
    files = ["/foo/junk", "/bar/junk"]
    actual = ExecutionContextMeta.identify(files)
    assert actual == (None, None), actual



"""
X86PageBranch - combiner for x86 kernel features:
=================================================

x86 kernel features includes:
    * PTI (Page Table Isolation)
    * IBPB (Indirect Branch Prediction Barrier)
    * IBRS (Indirect Branch Restricted Speculation)

This combiner reads information from debugfs:

Examples:
    >>> type(dv)
    <class 'insights.combiners.x86_page_branch.X86PageBranch'>
    >>> dv.pti
    1
    >>> dv.ibpb
    3
    >>> dv.ibrs
    2
    >>> dv.retp
    0

    Attributes:
        pti (int): The result parsed of '/sys/kernel/debug/x86/pti_enabled'

        ibpb (int): The result parsed of '/sys/kernel/debug/x86/ibpb_enabled'

        ibrs (int): The result parsed of '/sys/kernel/debug/x86/ibrs_enabled'

        retp (int): The result parsed of '/sys/kernel/debug/x86/retp_enabled'
"""

from insights.core.plugins import combiner
from insights.parsers.x86_debug import X86PTIEnabled, X86IBPBEnabled, X86IBRSEnabled, X86RETPEnabled


@combiner(X86PTIEnabled, X86IBPBEnabled, X86IBRSEnabled, optional=[X86RETPEnabled])
class X86PageBranch(object):
    """
    This combiner provides an interface to the three X86 Page Table/Branch Prediction parsers.
    If retp_enabled is not available, self.retp is None.
    """
    def __init__(self, pti_enabled, ibpb_enabled, ibrs_enabled, retp_enabled):
        self.pti = pti_enabled.value
        self.ibpb = ibpb_enabled.value
        self.ibrs = ibrs_enabled.value
        self.retp = None
        if retp_enabled:
            self.retp = retp_enabled.value



"""
LsEtc - command ``ls -lan /etc </etc/sub-dirs>``
================================================

The ``ls -lan /etc </etc/sub-directories>`` command provides information for
the listing of the ``/etc`` directory and specified sub-directories.
See ``FileListing`` class for additional information.

Sample ``ls -lan /etc/sysconfig /etc/rc.d/rc3.d`` output::

    /etc/sysconfig:
    total 96
    drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
    drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
    drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
    drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
    -rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
    -rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
    lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

    /etc/rc.d/rc3.d:
    total 4
    drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
    drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
    lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
    lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
    lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd

Examples:

    >>> "sysconfig" in ls_etc
    False
    >>> "/etc/sysconfig" in ls_etc
    True
    >>> len(ls_etc.files_of('/etc/sysconfig'))
    3
    >>> ls_etc.files_of("/etc/sysconfig")
    ['ebtables-config', 'firewalld', 'grub']
    >>> ls_etc.dirs_of("/etc/sysconfig")
    ['.', '..', 'cbq', 'console']
    >>> ls_etc.specials_of("/etc/sysconfig")
    []
    >>> ls_etc.total_of("/etc/sysconfig")
    96
    >>> ls_etc.dir_entry('/etc/sysconfig', 'grub') == {'group': '0', 'name': 'grub', 'links': 1, 'perms': 'rwxrwxrwx.', 'raw_entry': 'lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub', 'owner': '0', 'link': '/etc/default/grub', 'date': 'Jul  6 23:32', 'type': 'l', 'dir': '/etc/sysconfig', 'size': 17}
    True
    >>> ls_etc.files_of('/etc/rc.d/rc3.d')
    ['K50netconsole', 'S10network', 'S97rhnsd']
    >>> sorted(ls_etc.listing_of("/etc/sysconfig").keys()) == sorted(['console', 'grub', '..', 'firewalld', '.', 'cbq', 'ebtables-config'])
    True
    >>> sorted(ls_etc.listing_of("/etc/sysconfig")['console'].keys()) == sorted(['group', 'name', 'links', 'perms', 'raw_entry', 'owner', 'date', 'type', 'dir', 'size'])
    True
    >>> ls_etc.listing_of("/etc/sysconfig")['console']['type']
    'd'
    >>> ls_etc.listing_of("/etc/sysconfig")['console']['perms']
    'rwxr-xr-x.'
    >>> ls_etc.dir_contains("/etc/sysconfig", "console")
    True
    >>> ls_etc.dir_entry("/etc/sysconfig", "console") == {'group': '0', 'name': 'console', 'links': 2, 'perms': 'rwxr-xr-x.', 'raw_entry': 'drwxr-xr-x.  2 0 0    6 Sep 16  2015 console', 'owner': '0', 'date': 'Sep 16  2015', 'type': 'd', 'dir': '/etc/sysconfig', 'size': 6}
    True
    >>> ls_etc.dir_entry("/etc/sysconfig", "grub")['type']
    'l'
    >>> ls_etc.dir_entry("/etc/sysconfig", "grub")['link']
    '/etc/default/grub'
"""
from .. import parser, CommandParser
from .. import FileListing
from insights.specs import Specs


@parser(Specs.ls_etc)
class LsEtc(CommandParser, FileListing):
    """
    Parses output of ``ls -lan /etc </etc/sub-directories>`` command.
    """
    pass



from insights.tests import context_wrap
from insights.parsers.rhn_schema_version import rhn_schema_version

schema_content_ok = """
5.6.0.10-2.el6sat
""".strip()

schema_content_no = """
-bash: /usr/bin/rhn-schema-version: No such file or directory
""".strip()


def test_rhn_schema_version():
    result = rhn_schema_version(context_wrap(schema_content_ok))
    assert result == "5.6.0.10-2.el6sat"
    result = rhn_schema_version(context_wrap(schema_content_no))
    assert result is None



import string
from insights.parsr import (AnyChar, Choice, EOF, EOL, Forward, LeftCurly,
        LineEnd, Literal, Many, Number, OneLineComment, Opt, PosMarker,
        QuotedString, RightCurly, skip_none, String, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section


def loads(data):
    return Entry(children=Top(data)[0])


def load(f):
    return loads(f.read())


def to_entries(x):
    ret = []
    for i in x:
        name, attrs, body = i
        if body:
            for n in [name.value] + attrs:
                ret.append(Section(name=n, children=body, lineno=name.lineno))
        else:
            attrs = [attrs] if not isinstance(attrs, list) else attrs
            ret.append(Directive(name=name.value, attrs=attrs, lineno=name.lineno))
    return ret


scripts = set("postrotate prerotate firstaction lastaction preremove".split())
Stanza = Forward()
Spaces = Many(WSChar)
Bare = String(set(string.printable) - (set(string.whitespace) | set("#{}'\"")))
Num = Number & (WSChar | LineEnd)
Comment = OneLineComment("#").map(lambda x: None)
ScriptStart = WS >> PosMarker(Choice([Literal(s) for s in scripts])) << WS
ScriptEnd = Literal("endscript")
Line = (WS >> AnyChar.until(EOL) << WS).map(lambda x: "".join(x))
Lines = Line.until(ScriptEnd).map(lambda x: "\n".join(x))
Script = ScriptStart + Lines << ScriptEnd
Script = Script.map(lambda x: [x[0], x[1], None])
BeginBlock = WS >> LeftCurly << WS
EndBlock = WS >> RightCurly
First = PosMarker((Bare | QuotedString)) << Spaces
Attr = Spaces >> (Num | Bare | QuotedString) << Spaces
Rest = Many(Attr)
Block = BeginBlock >> Many(Stanza).map(skip_none).map(to_entries) << EndBlock
Stmt = WS >> (Script | (First + Rest + Opt(Block))) << WS
Stanza <= WS >> (Stmt | Comment) << WS
Doc = Many(Stanza).map(skip_none).map(to_entries)
Top = Doc + EOF
